import re

from risc_generator import NEW_PAGE_TAG, FPQ1Contract
from risc_generator.content.fr.fpq_1 import (
    generate_toc_text,
    generate_introduction_text,
    generate_declarations_text,
    generate_section_a_text,
    generate_section_b_text,
    generate_general_conditions_text,
    generate_loss_and_claims_reporting_text,
    generate_dates_text,
    generate_cancellation_table_text,
    generate_definitions_text,
)
from tests.domain.contract.contract_base import ContractTestBase


class ContractFPQ1Test(ContractTestBase):
    def test_generate_fpq1(self):
        fpq1_contract = FPQ1Contract(
            client_number=self.a_client_number,
            contract_start_date=self.a_contract_start_date,
            insuree=self.a_insuree,
            vehicle=self.a_vehicle,
            payment_method=self.a_pre_authorize_payment_method,
            protections=self.a_protections,
            premium=self.a_premium,
            financing=self.a_creditor,
            locale=self.a_fr_locale,
        )

        actual_fpq1 = fpq1_contract.generate_fpq1_text()

        actual_number_of_pages = len(re.findall("<###NEW_PAGE###>", actual_fpq1))

        expected = 35

        self.assertEqual(expected, actual_number_of_pages)

    def test_generation_toc(self):
        actual = generate_toc_text()
        expected = (
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "TABLE DES MATIÈRES\n"
            "INTRODUCTION 4\n"
            "1. DOCUMENTS INCLUS DANS LE CONTRAT D'ASSURANCE 4\n"
            "2. OBLIGATION D'INFORMER L'ASSUREUR 4\n"
            "CONDITIONS PARTICULIÈRES 5\n"
            "CHAPITRE A : GARANTIES POUR LA RESPONSABILITÉ CIVILE DÉCOULANT DES DOMMAGES MATÉRIELS ET DES "
            "DOMMAGES CORPORELS CAUSÉS À D'AUTRES PERSONNES (ASSURANCE OBLIGATOIRE) 7\n"
            "1. PERSONNES ASSURÉES 7\n"
            "2. VÉHICULES ASSURÉS 7\n"
            "3. GARANTIE PRINCIPALE 7\n"
            "3.1 Description de la garantie principale 7\n"
            "3.2 Précisions quant aux dommages 8\n"
            "4. GARANTIES ADDITIONNELLES 8\n"
            "4.1 Prise en charge et défense des intérêts des personnes assurées 8\n"
            "4.2 Prise en charge de certains frais liés à une poursuite 8\n"
            "4.3 Remboursement de soins médicaux 9\n"
            "4.4 Prise en charge des frais réclamés par une municipalité 9\n"
            "5. EXCLUSIONS 9\n"
            "6. INDEMNITÉ PAYABLE PAR L'ASSUREUR 9\n"
            "6.1 Règle générale 9\n"
            "6.2 Ajustement du montant d'assurance en raison de la loi 10\n"
            "6.3 Cas où les personnes assurées doivent rembourser l'indemnité 10\n"
            "6.4 Limitation du montant d'assurance pour le risque nucléaire 10\n"
            "6.5 Règles particulières pour le véhicule dont l'assuré désigné est nouvellement propriétaire 10\n"
            "6.6 Règles particulières pour le véhicule de remplacement temporaire 10\n"
            "6.7 Règles particulières pour le véhicule dont l'assuré désigné n'est pas propriétaire 11\n"
            "6.8 Règles particulières pour les remorques ou les semi-remorques attelées à un véhicule automobile 11\n"
            "6.9 Règles particulières pour le véhicule assuré confié à une personne qui exerce une activité "
            "professionnelle de garagiste 11\n"
            "7. MANDAT DE REPRÉSENTATION 12\n"
            "CHAPITRE B : GARANTIES POUR LES DOMMAGES AUX VÉHICULES ASSURÉS (ASSURANCE OPTIONNELLE) 12\n"
            "1. PERSONNE ASSURÉE 12\n"
            "2. VÉHICULES ASSURÉS 12\n"
            "3. GARANTIE PRINCIPALE 12\n"
            "3.1 Description de la garantie principale 12\n"
            "3.2 Description des protections 13\n"
            "4. GARANTIES ADDITIONNELLES 14\n"
            "4.1 Frais de déplacement en cas de vol d'un véhicule assuré 14\n"
            "4.2 Prise en charge des frais réclamés par une municipalité 14\n"
            "4.3 Prise en charge d'autres frais 15\n"
            "5. GARANTIE POUR LES VÉHICULES DE REMPLACEMENT TEMPORAIRE 15\n"
            "6. EXCLUSIONS 15\n"
            "7. FRANCHISE PAYABLE PAR L'ASSURÉ DÉSIGNÉ 17\n"
            "8. INDEMNITÉ PAYABLE PAR L'ASSUREUR 17\n"
            "8.1 Règle générale 17\n"
            "8.2 Délais pour le paiement de l'indemnité 17\n"
            "8.3 Règles particulières pour le véhicule dont l'assuré désigné est nouvellement propriétaire 17\n"
            "8.4 Règles particulières pour le véhicule de remplacement temporaire 17\n"
            "8.5 Règles particulières pour les remorques ou les semi-remorques attelées à un véhicule automobile 18\n"
            f"{NEW_PAGE_TAG}\n"
            "CONDITIONS GÉNÉRALES 18\n"
            "1. LOIS APPLICABLES AU CONTRAT D'ASSURANCE 18\n"
            "2. ENDROITS OÙ LES GARANTIES S'APPLIQUENT 18\n"
            "3. CONTINUATION DES GARANTIES APRÈS UN SINISTRE 18\n"
            "4. RÈGLES D'APPLICATION DU CONTRAT D'ASSURANCE LORSQU'IL Y A PLUSIEURS VÉHICULES DÉSIGNÉS 19\n"
            "5. INFORMATIONS À DÉCLARER À L'ASSUREUR 19\n"
            "5.1 Déclaration initiale du risque 19\n"
            "5.2 Aggravation du risque 19\n"
            "5.3 Conséquences en cas de fausses déclarations ou d'informations non déclarées 20\n"
            "6. NON-RESPECT D'UN ENGAGEMENT FORMEL 21\n"
            "7. USAGES INTERDITS D'UN VÉHICULE ASSURÉ 21\n"
            "8. EXAMEN DES VÉHICULES ASSURÉS 21\n"
            "9. ENVOI DES AVIS PAR L'ASSUREUR ET L'ASSURÉ DÉSIGNÉ 21\n"
            "DÉCLARER UN SINISTRE ET FAIRE UNE RÉCLAMATION 22\n"
            "1. QUOI FAIRE LORS D'UN SINISTRE 22\n"
            "1.1 Déclarer certaines informations à l'assureur 22\n"
            "1.2 Respecter certaines obligations relatives au véhicule assuré 23\n"
            "1.3 Ne pas se prononcer sur sa responsabilité et ne pas régler la réclamation 24\n"
            "1.4 Collaborer avec l'assureur 24\n"
            "2. COMMENT SE DÉTERMINE LA VALEUR DES DOMMAGES 24\n"
            "2.1 Valeur des dommages payable par l'assureur pour la réparation du véhicule assuré 24\n"
            "2.2 Valeur des dommages payable par l'assureur pour le véhicule assuré qui est une perte totale 25\n"
            "2.3 Cas où l'assureur peut décider de faire réparer, reconstruire ou remplacer le bien endommagé 25\n"
            "3. DROIT DE L'ASSUREUR APRÈS AVOIR PAYÉ UNE INDEMNITÉ (DROIT DE SUBROGATION) 25\n"
            "3.1 Règle générale 25\n"
            "3.2 Exceptions 25\n"
            "4. ARBITRAGE EN CAS DE DÉSACCORD ENTRE L'ASSURÉ DÉSIGNÉ ET L'ASSUREUR 26\n"
            "4.1 Faire une demande d'arbitrage 26\n"
            "4.2 Choisir les experts et l'arbitre 26\n"
            "4.3 Valeur des dommages payable par l'assureur 27\n"
            "4.4 Déroulement de l'arbitrage 27\n"
            "4.5 Choix de la langue 27\n"
            "4.6 Endroit où se déroule l'arbitrage 27\n"
            "4.7 Décision de l'arbitre 27\n"
            "4.8 Frais et honoraires de l'arbitrage 28\n"
            "5. MAINTIEN DES DROITS DE L'ASSURÉ DÉSIGNÉ ET DE L'ASSUREUR 28\n"
            "6. DÉLAI POUR ENTREPRENDRE UNE ACTION DÉCOULANT DU CONTRAT D'ASSURANCE (DÉLAI DE PRESCRIPTION) 28\n"
            "PRISE D'EFFET, RENOUVELLEMENT ET EXPIRATION DU CONTRAT D'ASSURANCE 28\n"
            "1. PRISE D'EFFET ET EXPIRATION DU CONTRAT D'ASSURANCE 28\n"
            "2. RENOUVELLEMENT DU CONTRAT D'ASSURANCE 28\n"
            "3. RÉSILIATION DU CONTRAT D'ASSURANCE (METTRE FIN AU CONTRAT D'ASSURANCE) 29\n"
            "3.1 Résiliation par l'assuré désigné 29\n"
            "3.2 Résiliation par l'assureur 29\n"
            "TABLEAU DE RÉSILIATION 31\n"
            "DÉFINITIONS 32\n"
            f"{NEW_PAGE_TAG}\n"
        )

        self.assertEqual(expected, actual)

    def test_generation_introduction(self):
        actual = generate_introduction_text()
        expected = (
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "INTRODUCTION\n"
            "L'introduction contient des explications générales sur le contrat d'assurance pour en faciliter la "
            "compréhension. Ces explications ne peuvent pas servir à créer un droit ou une garantie.\n"
            "En cas d'ambiguïté ou de divergence entre l'introduction et les lois applicables au contrat d'assurance, "
            "c'est le texte de ces lois qui a priorité.\n"
            "1. DOCUMENTS INCLUS DANS LE CONTRAT D'ASSURANCE\n"
            "Les documents suivants font partie du contrat d'assurance :\n"
            "\t Le présent document, à savoir le « Formulaire de police d'assurance automobile du Québec "
            "(F.P.Q.) N 1 – Formulaire des propriétaires ». Cette police d'assurance est un document standard "
            "approuvé par l'Autorité des marchés financiers.\n"
            "À noter que la section « Conditions particulières » de cette police d'assurance contient des "
            "informations spécifiques à la situation de l'assuré désigné."
            "\t Les avenants nommés à l'article 4 de la section « Conditions particulières ».\n"
            "Voici quelques indications utiles pour comprendre le contrat d'assurance :\n"
            "\t Se référer à la « Table des matières » pour comprendre la structure du contrat d'assurance et "
            "pour trouver une information en particulier.\n"
            "\t Les mots et les expressions en caractère gras dans le présent document et dans les avenants "
            "sont expliqués à la section « Définitions ». À noter que les avenants peuvent comporter leurs "
            "propres définitions.\n"
            "\t Le contrat d'assurance doit être lu comme un tout. Les clauses doivent donc être interprétées les "
            "unes par rapport aux autres, d'après le sens qui tient compte de l'ensemble du contrat "
            "d'assurance.\n"
            "\t Les garanties du chapitre A et du chapitre B sont des garanties différentes qui s'appliquent "
            "indépendamment l'une de l'autre.\n"
            "\t L'utilisation du singulier inclut le pluriel.\n"
            "2. OBLIGATION D'INFORMER L'ASSUREUR\n"
            "Tant avant la conclusion du contrat d'assurance que pendant sa durée, toutes les informations qui "
            "peuvent influencer le risque doivent être déclarées à l'assureur. Elles doivent aussi être déclarées lors "
            "du renouvellement du contrat d'assurance.\n"
            "En cas de doute sur l'obligation de déclarer une information en particulier, il est préférable de "
            "communiquer avec l'assureur.\n"
            "Entre autres, les informations suivantes doivent être déclarées :\n"
            "\t Tout changement dans l'usage du véhicule désigné.\n"
            "\t Si des personnes autres que l'assuré désigné conduisent le véhicule désigné.\n"
            "\t Tout accident automobile ou tout sinistre survenu dans le passé.\n"
            "\t Toute condamnation pour une infraction au Code de la sécurité routière.\n"
            "\t Toute condamnation criminelle.\n"
            "\t Toute modification ou ajout au véhicule désigné.\n"
            "\t Tout changement à la situation personnelle de l'assuré désigné ou à celle des conducteurs.\n"
            "L'obligation d'informer l'assureur est décrite en détail à l'article 5 de la section "
            "« Conditions générales ».\n"
            f"{NEW_PAGE_TAG}\n"
        )

        self.assertEqual(expected, actual)

    def test_generation_declarations(self):
        actual = generate_declarations_text()
        expected = (
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "CONDITIONS PARTICULIÈRES\n"
            "ARTICLE 1\n"
            "Nom et adresse de l'assuré désigné :\n"
            "La ville et la province de l'adresse écrite à cet article 1 constituent les lieux d'usage principal, de "
            "remisage et de stationnement du véhicule désigné. Si ce n'est pas le cas, le preneur ou l'assuré "
            "désigné doit le déclarer.\n"
            "ARTICLE 2\n"
            "Durée du contrat :\n"
            "Du ________________* au _______________* exclusivement.\n"
            "*à 0 h 01 selon l'heure normale à l'adresse de l'assuré désigné.\n"
            "ARTICLE 3\n"
            "Caractéristiques du véhicule désigné :\n"
            "Créancier qui a droit aux indemnités du chapitre B, selon son intérêt :\n"
            "ARTICLE 4\n"
            "Les risques couverts par le contrat d'assurance sont ceux pour lesquels un montant d'assurance, une "
            "franchise ou une prime d'assurance est écrit au tableau ci-dessous. Ils sont couverts aux conditions "
            "énoncées dans le contrat d'assurance.\n"
            "GARANTIES | RISQUES | MONTANT D'ASSURANCE ET FRANCHISES | PRIMES D'ASSURANCE\n"
            "Chapitre A : Responsabilité civile | Dommages matériels ou dommages corporels causés à "
            "d'autres personnes | Montant d'assurance :  $ |  $\n"
            "Chapitre B : Dommages aux véhicules assurés | Protection 1 : « Tous risques », Protection 2 : "
            "Risques de collision et de renversement, Protection 3 : Tous les risques sauf collision ou renversement, "
            "Protection 4 : Risques spécifiques | Franchise par sinistre : $ $ $ $ | $ $ $ $\n"
            "Avenants : |  |  |  $|\n"
            "Date limite pour le paiement des primes d'assurance : | Total: | $\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            f"ARTICLE 5\n"
            f"L'assuré désigné est le propriétaire réel et le titulaire de l'immatriculation du véhicule désigné."
            f"Si ce n'est pas le cas, les informations suivantes doivent être déclarées :\n"
            f"Propriétaire réel :\n"
            f"Titulaire de l'immatriculation :\n"
            f"ARTICLE 6\n"
            f"Déclarations importantes pour l'analyse du risque :\n"
            f"ARTICLE 7\n"
            f"Informations pour l'assuré désigné :\n"
            f"Nom de l'agent ou du courtier en assurance :\n"
            f"Adresse de l'agent ou du courtier en assurance :\n"
            f"{NEW_PAGE_TAG}\n"
        )

        self.assertEqual(expected, actual)

    def test_section_a(self):
        actual = generate_section_a_text()
        expected = (
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "CHAPITRE A :\n"
            "GARANTIES POUR LA RESPONSABILITÉ CIVILE DÉCOULANT DES DOMMAGES MATÉRIELS ET DES DOMMAGES CORPORELS CAUSÉS "
            "À D'AUTRES PERSONNES\n"
            "(ASSURANCE OBLIGATOIRE)\n"
            "1. PERSONNES ASSURÉES\n"
            "Les personnes assurées au chapitre A sont les suivantes :\n"
            "\t l'assuré désigné;\n"
            "\t toute personne qui conduit un véhicule assuré;\n"
            "\t toute personne qui fait usage d'un véhicule assuré. Toute personne qui fait fonctionner une "
            "partie d'un véhicule assuré est réputée en faire usage.\n"
            "Les représentants légaux et la succession de ces personnes sont également assurés.\n"
            "La personne qui vole un véhicule assuré ou qui est complice du vol n'est pas assurée.\n"
            "2. VÉHICULES ASSURÉS\n"
            "Sauf si le contexte indique un sens différent, l'expression « véhicule assuré » utilisée au chapitre A "
            "fait référence aux véhicules suivants :\n"
            "\t A. Véhicule désigné.\n"
            "\t B. Véhicule dont l'assuré désigné est nouvellement propriétaire, aux conditions énoncées à "
            "sa définition.\n"
            "\t C. Véhicule de remplacement temporaire, aux conditions énoncées à sa définition.\n"
            "\t D. Véhicule dont l'assuré désigné n'est pas propriétaire, aux conditions énoncées à sa définition.\n"
            "\t E. Remorque ou semi-remorque dont l'assuré désigné est propriétaire, aux conditions "
            "énoncées à sa définition.\n"
            "\t F. Remorque ou semi-remorque dont l'assuré désigné n'est pas propriétaire et qui est utilisée "
            "avec un véhicule assuré au contrat d'assurance.\n"
            "3. GARANTIE PRINCIPALE\n"
            "3.1 Description de la garantie principale\n"
            "Le chapitre A couvre le risque suivant : les conséquences financières que peut subir une personne "
            "assurée lorsqu'elle est civilement responsable d'un dommage causé à une autre personne par un "
            "véhicule assuré.\n"
            "La responsabilité civile de la personne assurée doit découler du fait qu'elle est la propriétaire du "
            "véhicule ou du fait qu'elle le conduisait ou en faisait usage.\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "3.2 Précisions quant aux dommages\n"
            "3.2.1 Dommages occasionnés aux remorques ou semi-remorques dont les personnes "
            "assurées ne sont pas propriétaires\n"
            "Lorsqu'un dommage est occasionné à une remorque ou une semi-remorque dont les "
            "personnes assurées ne sont pas propriétaires et :\n"
            "\t que la remorque ou la semi-remorque est attelée à un véhicule automobile "
            "utilisé à des fins personnelles qui est assuré par le présent chapitre A; ou\n"
            "\t qu'elle n'est pas attelée à un tel véhicule à la condition qu'elle le soit "
            "habituellement;\n"
            "les conséquences financières subies par les personnes assurées sont couvertes si la "
            "remorque ou la semi-remorque n'est pas conçue ni utilisée pour le transport de "
            "personnes, ni à des fins de démonstration, de vente, de bureau, ou d'habitation.\n"
            "3.2.2 Dommages visés par la Convention d'indemnisation directe\n"
            "Lorsqu'un dommage matériel est causé à une personne assurée et que la Convention "
            "d'indemnisation directe établie conformément à la Loi sur l'assurance automobile "
            "s'applique, ce dommage est couvert par le chapitre A.\n"
            "3.2.3 Dommages causés à un autre assuré désigné\n"
            "Lorsque c'est un assuré désigné qui subit un dommage du fait d'un autre assuré "
            "désigné, celui qui subit le dommage est considéré comme une autre personne. Il peut "
            "donc être indemnisé par l'assureur au chapitre A.\n"
            "4. GARANTIES ADDITIONNELLES\n"
            "Lorsque la garantie principale s'applique, le chapitre A inclut les garanties additionnelles suivantes :\n"
            "4.1 Prise en charge et défense des intérêts des personnes assurées\n"
            "Lorsqu'un sinistre lui est déclaré, l'assureur prend en charge les intérêts des personnes "
            "assurées et assume leur défense.\n"
            "Il agit comme il le veut en matière d'enquête, de transaction ou de règlement.\n"
            "Par contre, l'assureur ne peut pas invoquer un moyen de défense qui est interdit aux "
            "assureurs de l'endroit où le sinistre est survenu.\n"
            "4.2 Prise en charge de certains frais liés à une poursuite\n"
            "Lorsqu'une personne assurée est poursuivie, l'assureur prend en charge :\n"
            "\t les frais et les dépens qui découlent de cette poursuite; et\n"
            "\t les intérêts sur le montant d'assurance.\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "4.3 Remboursement de soins médicaux\n"
            "Lorsqu'une autre personne subit un dommage corporel, l'assureur rembourse les dépenses "
            "engagées par une personne assurée pour les soins médicaux immédiatement nécessaires.\n"
            "4.4 Prise en charge des frais réclamés par une municipalité\n"
            "L'assureur prend en charge les frais réclamés à l'assuré désigné par une municipalité en "
            "vertu de la Loi sur la fiscalité municipale et ses règlements, lorsque son service de sécurité "
            "incendie est intervenu pour prévenir ou combattre l'incendie d'un véhicule assuré.\n"
            "5. EXCLUSIONS\n"
            "Les exclusions suivantes s'appliquent au chapitre A :\n"
            "A. Tout dommage corporel dont l'indemnisation est prévue dans l'une des lois suivantes :\n"
            "\t la Loi sur l'assurance automobile;\n"
            "\t la Loi sur les accidents du travail et les maladies professionnelles;\n"
            "\t la Loi sur l'indemnisation des victimes d'actes criminels.\n"
            "Cette exclusion ne s'applique pas si la Loi sur l'assurance automobile n'est pas applicable.\n"
            "B. Toute responsabilité imposée par une législation visant les accidents du travail.\n"
            "C. Tout dommage corporel causé à une personne qui est employée par une personne assurée "
            "et qui est occupée à faire fonctionner ou à réparer un véhicule assuré.\n"
            "D. Tout dommage causé à une personne qui exerce une activité professionnelle de garagiste, "
            "pendant que le véhicule assuré lui est confié.\n"
            "E. Le sinistre qui survient pendant que le véhicule assuré:\n"
            "\t est loué à une autre personne;\n"
            "\t est utilisé comme taxi, véhicule de visites touristiques, autobus, autocar ou comme tout "
            "autre véhicule fourni avec chauffeur;\n"
            "\t transporte des explosifs;\n"
            "\t transporte des substances radioactives à des fins de recherche, d'éducation, "
            "d'expansion, d'industrie, ou à d'autres fins connexes.\n"
            "L'assureur peut accepter de couvrir ces situations en les spécifiant à la section « Conditions "
            "particulières » ou par un avenant.\n"
            "6. INDEMNITÉ PAYABLE PAR L'ASSUREUR\n"
            "6.1 Règle générale\n"
            "L'indemnité payable par l'assureur ne peut pas dépasser le montant d'assurance, auquel "
            "s'ajoutent les frais couverts par les garanties additionnelles. Cette règle s'applique même :\n"
            "\t s'il y a plusieurs personnes assurées ou une multiplicité d'intérêts;\n"
            "\t si plusieurs personnes subissent un dommage;\n"
            "\t si plusieurs personnes assurées sont civilement responsables des dommages "
            "découlant d'un même sinistre; et\n"
            "\t si les dommages sont de différentes natures.\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "Si plusieurs personnes assurées sont civilement responsables des dommages découlant d'un "
            "même sinistre et que le montant d'assurance est insuffisant, l'assuré désigné bénéficie des "
            "garanties en priorité.\n"
            "6.2 Ajustement du montant d'assurance en raison de la loi\n"
            "Si le montant d'assurance est inférieur au montant minimum exigé par les lois relatives à "
            "l'assurance des véhicules automobiles en vigueur à l'endroit du sinistre, le montant "
            "d'assurance sera ajusté pour respecter ce minimum.\n"
            "6.3 Cas où les personnes assurées doivent rembourser l'indemnité\n"
            "Si l'assureur doit indemniser une autre personne en raison d'une disposition légale visant "
            "l'assurance de véhicules automobiles, et qu'il n'était pas tenu de le faire en vertu du contrat "
            "d'assurance, les personnes assurées s'engagent à rembourser cette indemnité à l'assureur, à "
            "sa demande.\n"
            "6.4 Limitation du montant d'assurance pour le risque nucléaire\n"
            "Si c'est la réalisation d'un risque nucléaire qui a occasionné les dommages, le montant "
            "d'assurance applicable se limite au montant minimum imposé par l'une des lois suivantes, "
            "selon le type de véhicule automobile impliqué dans le sinistre :\n"
            "\t la Loi sur l'assurance automobile;\n"
            "\t la Loi sur les véhicules hors route.\n"
            "6.5 Règles particulières pour le véhicule dont l'assuré désigné est nouvellement propriétaire\n"
            "A. Le véhicule dont l'assuré désigné est nouvellement propriétaire bénéficie des mêmes "
            "garanties que le véhicule désigné.\n"
            "B. Si l'assuré désigné a plusieurs véhicules désignés assurés avec l'assureur, en vertu "
            "d'un ou plusieurs contrats d'assurance, le véhicule dont il est nouvellement propriétaire "
            "bénéficie de la moindre des garanties déjà accordées sur tous ces autres véhicules.\n"
            "6.6 Règles particulières pour le véhicule de remplacement temporaire\n "
            "A. Tout contrat d'assurance de responsabilité civile établi au nom du propriétaire d'un "
            "véhicule de remplacement temporaire s'applique en premier.\n"
            "B. Le présent chapitre A intervient seulement si l'assurance de ce propriétaire est "
            "insuffisante, et ce :\n"
            "\t jusqu'à concurrence du montant d'assurance du véhicule désigné; et\n"
            "\t uniquement pour ce qui excède l'obligation de l'assureur du propriétaire.\n"
            "Si l'assuré désigné a plusieurs véhicules désignés assurés avec l'assureur, en vertu "
            "d'un ou plusieurs contrats d'assurance, c'est le plus élevé des montants d'assurance de "
            "ces véhicules qui s'applique.\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "6.7 Règles particulières pour le véhicule dont l'assuré désigné n'est pas propriétaire\n"
            "A. Tout contrat d'assurance de responsabilité civile établi au nom du propriétaire d'un "
            "véhicule dont l'assuré désigné n'est pas propriétaire s'applique en premier.\n"
            "B. Le présent chapitre A intervient seulement si l'assurance de ce propriétaire est "
            "insuffisante, et ce :\n"
            "\t jusqu'à concurrence du montant d'assurance du véhicule désigné; et\n"
            "\t uniquement pour ce qui excède l'obligation de l'assureur du propriétaire.\n"
            "Si l'assuré désigné a plusieurs véhicules désignés assurés avec l'assureur, en vertu "
            "d'un ou plusieurs contrats d'assurance, c'est le plus élevé des montants d'assurance de"
            "ces véhicules qui s'applique.\n"
            "C. Dans les quatre cas d'exception énumérés à la définition du véhicule dont l'assuré "
            "désigné n'est pas propriétaire et aux conditions qui y sont énoncées, le montant "
            "d'assurance est limité au montant minimum exigé par l'une des lois suivantes, selon le "
            "type de véhicule automobile impliqué dans le sinistre :\n"
            "\t la Loi sur l'assurance automobile;\n"
            "\t la Loi sur les véhicules hors route.\n"
            "6.8 Règles particulières pour les remorques ou les semi-remorques attelées à un véhicule automobile\n"
            "A. Lorsqu'une ou plusieurs remorques ou semi-remorques sont attelées à un véhicule "
            "automobile, elles sont réputées constituer avec lui un seul et même véhicule.\n"
            "Cette règle signifie que si un dommage est causé par le véhicule automobile, la "
            "remorque ou la semi-remorque, un seul de leur montant d'assurance s'applique, soit le plus élevé des "
            "montants.\n"
            "B. La même règle s'applique si ces véhicules sont assurés par différents contrats d'assurance avec "
            "l'assureur.\n"
            "C. Dans les quatre cas d'exception énumérés à la définition du véhicule dont l'assuré "
            "désigné n'est pas propriétaire et aux conditions qui y sont énoncées, le montant "
            "d'assurance est limité au montant minimum exigé par l'une des lois suivantes, selon le "
            "type de véhicule automobile impliqué dans le sinistre :\n"
            "\t la Loi sur l'assurance automobile;\n"
            "\t la Loi sur les véhicules hors route.\n"
            "6.9 Règles particulières pour le véhicule assuré confié à une personne qui exerce "
            "une activité professionnelle de garagiste\n"
            "Lorsqu'un dommage est causé par un véhicule assuré qui fait l'objet d'une activité "
            "professionnelle de garagiste au moment du sinistre :\n "
            "\t A. C'est le contrat d'assurance de la personne à qui le véhicule est confié et qui exerce cette "
            "activité qui s'applique en premier, et ce, à la condition que ce contrat couvre sa "
            "responsabilité civile sans désigner expressément les véhicules qui sont assurés.\n"
            "\t B. Le présent chapitre A intervient seulement si l'assurance de cette personne est "
            "insuffisante, et ce :\n"
            "jusqu'à concurrence du montant d'assurance applicable; et\n"
            "uniquement pour ce qui excède l'obligation de l'assureur de cette personne.\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "7. MANDAT DE REPRÉSENTATION\n"
            "Les personnes assurées donnent à l'assureur le mandat de les représenter dans toute poursuite "
            "intentée contre elles au Canada ou aux États-Unis.\n"
            "La poursuite doit découler du fait qu'une personne assurée est propriétaire du véhicule assuré ou "
            "du fait qu'elle le conduisait ou en faisait usage.\n"
            "Ce mandat de représentation inclut, entre autres, le droit pour l'assureur de comparaître au nom "
            "des personnes assurées et d'assumer leur défense.\n"
            "Les personnes assurées renoncent à leur droit de retirer ce mandat à l'assureur sans son consentement.\n"
        )

        self.assertEqual(expected, actual)

    def test_generation_section_b(self):
        actual = generate_section_b_text()
        expected = (
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "CHAPITRE B :\n"
            "GARANTIES POUR LES DOMMAGES AUX VÉHICULES ASSURÉS\n"
            "(ASSURANCE OPTIONNELLE)\n"
            "1. PERSONNE ASSURÉE\n"
            "La personne assurée au chapitre B est l'assuré désigné.\n"
            "Pour connaître les personnes assurées lorsqu'un véhicule de remplacement temporaire est "
            "impliqué dans un sinistre, voir l'article 5 du chapitre B.\n"
            "2. VÉHICULES ASSURÉS\n"
            "Sauf si le contexte indique un sens différent, l'expression « véhicule assuré » utilisée au chapitre B "
            "fait référence aux véhicules suivants :\n"
            "A. Véhicule désigné.\n"
            "B. Véhicule dont l'assuré désigné est nouvellement propriétaire¸ aux conditions énoncées à "
            "sa définition.\n"
            "3. GARANTIE PRINCIPALE\n"
            "3.1 Description de la garantie principale\n"
            "Le chapitre B couvre :\n"
            "\t les dommages occasionnés directement et accidentellement à un véhicule assuré ou à "
            "ses équipements et accessoires; et\n"
            "\t la disparition d'un véhicule assuré ou de ses équipements et accessoires.\n"
            "Ces dommages ou cette disparition doivent résulter de la réalisation d'un risque couvert par "
            "la protection applicable.\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "3.2 Description des protections\n"
            "Pour connaître la protection applicable, voir l'article 4 de la section « Conditions particulières ».\n"
            "3.2.1 Protection 1 – Garantie « tous risques »\n"
            "Cette protection couvre les dommages occasionnés par tout type de risques. Par "
            "contre, elle exclut les risques et les dommages énumérés à l'article 6 du chapitre B.\n"
            "3.2.2 Protection 2 – Garantie contre les risques de collision et de renversement\n"
            "Cette protection couvre les dommages occasionnés par les risques de collision et de "
            "renversement.\n"
            "Une « collision » inclut, entre autres :\n"
            "\t toute collision entre un véhicule assuré et le sol;\n"
            "\t toute collision entre deux véhicules attelés l'un à l'autre; et\n"
            "\t toute collision entre un véhicule assuré et une personne ou un animal.\n"
            "Le « renversement » d'un véhicule assuré peut être partiel ou complet.\n"
            "Par contre, cette protection exclut les risques et les dommages énumérés à l'article 6 "
            "du chapitre B.\n"
            "3.2.3 Protection 3 - Garantie contre les risques qui ne sont pas une collision ou un renversement\n"
            "Cette protection couvre les dommages occasionnés par les risques qui ne sont pas "
            "une collision ou un renversement.\n "
            "Elle couvre, entre autres, les dommages occasionnés par les risques suivants :\n"
            "\t les risques énumérés à la Protection 4;\n"
            "\t les actes malveillants;\n"
            "\t les objets qui tombent ou qui volent;\n"
            "\t les projectiles;\n"
            "Tout comme la Protection 2, cette protection couvre aussi les dommages occasionnés "
            "par une collision entre un véhicule assuré et une personne ou un animal.\n"
            "Par contre, cette protection exclut les risques et les dommages énumérés à l'article 6 du chapitre B.\n"
            "3.2.4 Protection 4 - Garantie contre des risques spécifiques\n"
            "Cette protection couvre uniquement les dommages occasionnés par les risques suivants :\n"
            "\t l'atterrissage forcé ou la chute d'un aéronef ou d'une partie de cet appareil;\n"
            "\t la crue des eaux;\n"
            "\t l'échouement, la submersion, l'incendie, le déraillement ou la collision de tout "
            "véhicule terrestre ou bateau servant à transporter un véhicule assuré;\n"
            "\t les émeutes;\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "\t les explosions;\n"
            "\t la foudre;\n"
            "\t la grêle;\n"
            "\t l'incendie;\n"
            "\t les mouvements populaires;\n"
            "\t les tempêtes de vent;\n"
            "\t les tentatives de vol;\n"
            "\t les tremblements de terre;\n"
            "\t le vol.\n"
            "Par contre, cette protection exclut les risques et les dommages énumérés à l'article 6 du chapitre B.\n"
            "4. GARANTIES ADDITIONNELLES\n"
            "Lorsque la garantie principale s'applique, le chapitre B inclut les garanties additionnelles suivantes :\n"
            "4.1 Frais de déplacement en cas de vol d'un véhicule assuré\n"
            "Si un véhicule assuré est couvert par la Protection 1, la Protection 3 ou la Protection 4 et que "
            "l'assuré désigné ne peut plus l'utiliser parce qu'il a été volé en entier, l'assureur lui rembourse "
            "les frais suivants :\n"
            "\t les frais de location pour un véhicule de remplacement temporaire;\n"
            "\t les frais de taxi;\n"
            "\t les frais de transport en commun.\n"
            "Sur production des reçus de paiement, ces frais sont remboursés jusqu'à un montant maximum "
            "de 40 $ par jour et de 1 200 $ par sinistre.\n"
            "Cette garantie s'applique uniquement aux frais engagés à partir de 72 heures après la "
            "déclaration de vol à la police ou à l'assureur, et ce, même si le contrat d'assurance a expiré "
            "depuis le sinistre.\n"
            "Ces frais cessent d'être remboursés :\n"
            "\t lorsque le véhicule assuré est remplacé, ou réparé s'il a été retrouvé endommagé; ou\n"
            "\t lorsqu'une entente sur le règlement du sinistre est conclue avant que le véhicule "
            "assuré soit remplacé ou réparé.\n"
            "4.2 Prise en charge des frais réclamés par une municipalité\n"
            "L'assureur prend en charge les frais réclamés à l'assuré désigné par une municipalité en "
            "vertu de la Loi sur la fiscalité municipale et ses règlements, lorsque son service de sécurité "
            "incendie est intervenu pour prévenir ou combattre l'incendie d'un véhicule assuré.\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "4.3 Prise en charge d'autres frais\n"
            "L'assureur prend en charge les frais suivants lorsque l'assuré désigné en est civilement "
            "responsable :\n"
            "\t les frais d'avarie commune;\n"
            "\t les frais de sauvetage;\n"
            "\t les droits de douane du Canada et des États-Unis.\n"
            "5. GARANTIE POUR LES VÉHICULES DE REMPLACEMENT TEMPORAIRE\n"
            "L'assureur garantit l'assuré désigné et le conducteur d'un véhicule de remplacement temporaire "
            "contre les conséquences financières qu'ils peuvent subir lorsqu'ils sont civilement responsables du "
            "fait :\n"
            "\t d'un dommage direct et accidentel causé à ce véhicule; ou\n"
            "\t de la disparition de ce véhicule.\n"
            "La responsabilité civile peut être contractuelle ou extracontractuelle.\n"
            "Cette garantie s'applique aux conditions suivantes :\n"
            "\t la protection applicable au véhicule de remplacement temporaire, soit celle du véhicule "
            "désigné qu'il remplace, doit couvrir le risque ayant causé le dommage; et\n"
            "\t l'assuré désigné ou le conducteur doit avoir un pouvoir de direction ou de gestion sur le "
            "véhicule ou en avoir la garde.\n"
            "Les garanties additionnelles du chapitre B ne s'appliquent pas à cette garantie. Par contre, si le cas "
            "se présente, les garanties additionnelles du chapitre A peuvent s'appliquer.\n"
            "Les exclusions énumérées à l'article 6 du chapitre B sont applicables à cette garantie.\n"
            "L'indemnité payable par l'assureur est déterminée selon les règles énoncées à l'article 8 du chapitre B.\n"
            "6. EXCLUSIONS\n"
            "Les exclusions suivantes s'appliquent au chapitre B :\n"
            "A. Les dommages occasionnés aux pneus, sauf en cas :\n"
            "\t de coïncidence avec d'autres dommages couverts par la même garantie; ou\n"
            "\t d'incendie, de vol ou d'actes malveillants couverts par la même garantie.\n"
            "B. Les dommages occasionnés par :\n"
            "\t un bris mécanique;\n"
            "\t la corrosion;\n"
            "\t l'explosion dans les chambres de combustion;\n"
            "\t le gel;\n"
            "\t une panne;\n"
            "\t la rouille;\n"
            "\t l'usure normale.\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "Par contre, ces dommages ne sont pas exclus :\n"
            "\t en cas de coïncidence avec d'autres dommages couverts par la même garantie; ou\n"
            "\t en cas d'incendie, de vol ou d'actes malveillants couverts par la même garantie.\n"
            "C. Pour la Protection 3 et la Protection 4 seulement, le vol commis par l'une des personnes "
            "suivantes :\n"
            "\t une personne qui a le même domicile que celui de l'assuré désigné;\n"
            "\t une personne employée par l'assuré désigné en tant que préposé à la conduite, à la "
            "vente, à l'équipement, à la réparation, à l'entretien, au remisage, au stationnement, au "
            "déplacement ou au contrôle du bon fonctionnement d'un véhicule assuré, que cette "
            "personne soit ou non dans l'exercice de ses fonctions au moment du vol.\n"
            "D. L'appropriation illicite, le détournement, le vol ou le recel d'un véhicule assuré par une personne "
            "qui en a la possession légitime en vertu d'une hypothèque, d'une vente conditionnelle, d'un "
            "contrat de location, d'un contrat de crédit-bail ou en vertu de toute autre convention écrite "
            "similaire.\n"
            "E. L'aliénation volontaire des titres de propriété, avec ou sans abus de confiance, fraude ou "
            "déclaration mensongère.\n"
            "F. Le contenu d'une remorque ou d'une semi-remorque.\n"
            "G. Les rubans ou les accessoires de magnétophone, ou les disques compacts, à moins qu'ils ne "
            "soient en place dans un appareil.\n"
            "H. Qu'il y ait ou non déclaration de guerre, les dommages occasionnés, dans quelque mesure que "
            "ce soit, par :\n"
            "\t les activités des forces armées engagées dans des hostilités;\n"
            "\t les bombardements;\n"
            "\t la force militaire;\n"
            "\t la guerre civile;\n"
            "\t l'insurrection;\n"
            "\t l'invasion;\n"
            "\t la rébellion;\n"
            "\t la révolution;\n"
            "\t l'usurpation de pouvoir.\n"
            "I. Les dommages causés par un sinistre qui survient pendant que le véhicule assuré :\n"
            "\t est loué à une autre personne;\n"
            "\t est utilisé comme taxi, véhicule de visites touristiques, autobus, autocar ou comme tout "
            "autre véhicule fourni avec chauffeur;\n"
            "\t transporte des explosifs;\n"
            "\t transporte des substances radioactives à des fins de recherche, d'éducation, d'expansion, "
            "d'industrie, ou à d'autres fins connexes.\n"
            "L'assureur peut accepter de couvrir ces situations en les spécifiant à la section « Conditions "
            "particulières » ou par un avenant.\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "7. FRANCHISE PAYABLE PAR L'ASSURÉ DÉSIGNÉ\n"
            "L'assuré désigné doit assumer la franchise relative à la protection applicable. Par contre, si les "
            "dommages sont occasionnés par la foudre ou l'incendie, la franchise ne s'applique pas.\n"
            "Pour connaître la franchise applicable à chacune des protections, voir l'article 4 de la section "
            "« Conditions particulières » ou les avenants, selon le cas.\n"
            "8. INDEMNITÉ PAYABLE PAR L'ASSUREUR\n"
            "8.1 Règle générale\n"
            "L'indemnité payable par l'assureur correspond à la valeur des dommages, moins la "
            "franchise. Elle inclut aussi les frais couverts par les garanties additionnelles.\n"
            "Pour les règles relatives à la façon dont se détermine la valeur des dommages, voir l'article 2 "
            "de la section « Déclarer un sinistre et faire une réclamation ».\n"
            "8.2 Délais pour le paiement de l'indemnité\n"
            "L'assureur doit payer l'indemnité :\n"
            "\t dans les 60 jours qui suivent le moment où l'assuré désigné a déclaré le sinistre; ou\n"
            "\t dans les 60 jours qui suivent le moment où l'assureur a reçu les informations ou les "
            "pièces justificatives qu'il a exigées.\n"
            "8.3 Règles particulières pour le véhicule dont l'assuré désigné est nouvellement propriétaire\n"
            "A. Si le véhicule dont l'assuré désigné est nouvellement propriétaire remplace ou "
            "s'ajoute à un seul véhicule désigné :\n"
            "\t la protection applicable est la même que celle du véhicule désigné;\n"
            "\t la franchise applicable est la même que celle du véhicule désigné.\n"
            "B. Si le véhicule s'ajoute à plusieurs véhicules désignés assurés par un ou plusieurs "
            "contrats d'assurance avec l'assureur :\n"
            "\t le dommage est couvert seulement si tous les véhicules désignés sont assurés, "
            "au moment du sinistre, pour le risque ayant causé le dommage;\n"
            "\t la franchise applicable est la plus élevée de toutes les franchises des protections "
            "qui couvrent le risque ayant causé le dommage.\n"
            "8.4 Règles particulières pour le véhicule de remplacement temporaire\n"
            "A. Tout contrat d'assurance du propriétaire du véhicule de remplacement temporaire s'applique "
            "en premier.\n"
            "B. Le présent chapitre B s'applique seulement dans l'une des situations suivantes :\n"
            "\t le propriétaire de ce véhicule n'est pas assuré pour les dommages causés à son véhicule;\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "\t le propriétaire de ce véhicule est assuré, mais la franchise de son contrat "
            "d'assurance est plus élevée que celle du présent contrat. L'indemnité du présent "
            "chapitre B se limite alors à la différence entre :\n"
            "- la franchise du propriétaire de ce véhicule; et\n"
            "- la franchise du véhicule désigné qui est remplacé par le véhicule de "
            "remplacement temporaire.\n"
            "8.5 Règles particulières pour les remorques ou les semi-remorques attelées à un véhicule automobile\n"
            "Si une ou plusieurs remorques ou semi-remorques sont attelées à un véhicule automobile et "
            "que ces véhicules subissent un dommage lors d'un même sinistre :\n"
            "\t ces véhicules sont réputés être des véhicules distincts;\n"
            "\t ce sont donc leur propre protection et leur propre franchise qui s'appliquent.\n"
        )

        self.assertEqual(expected, actual)

    def test_generation_general_conditions(self):
        actual = generate_general_conditions_text()
        expected = (
            "CONDITIONS GÉNÉRALES\n"
            "1. LOIS APPLICABLES AU CONTRAT D'ASSURANCE\n"
            "Le contrat d'assurance est régi par les lois suivantes :\n"
            "\t le Code civil du Québec;\n"
            "\t le Code de procédure civile du Québec;\n"
            "\t la Loi sur l'assurance automobile et ses règlements; et\n"
            "\t la Loi sur les véhicules hors route lorsque le cas se présente.\n"
            "Certaines conditions générales du contrat d'assurance sont une version simplifiée des exigences de "
            "ces lois. En cas d'ambiguïté ou de divergence, c'est le texte de ces lois qui a priorité.\n"
            "2. ENDROITS OÙ LES GARANTIES S'APPLIQUENT\n"
            "Les garanties du contrat d'assurance s'appliquent uniquement lorsque le sinistre survient dans l'un "
            "des endroits suivants :\n"
            "\t au Canada ou aux États-Unis;\n"
            "dans un bateau ou dans un aéronef qui fait le service entre les ports et les aéroports de ces pays.\n"
            "L'assureur peut accepter, par un avenant, de couvrir des sinistres qui surviennent ailleurs.\n"
            "3. CONTINUATION DES GARANTIES APRÈS UN SINISTRE\n"
            "Un sinistre ne met pas fin au contrat d'assurance.\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "4. RÈGLES D'APPLICATION DU CONTRAT D'ASSURANCE LORSQU'IL Y A PLUSIEURS VÉHICULES DÉSIGNÉS\n"
            "S'il y a plusieurs véhicules désignés, le contrat d'assurance est réputé s'appliquer à chacun de ces "
            "véhicules comme si un contrat distinct avait été conclu pour chacun d'eux.\n"
            "5. INFORMATIONS À DÉCLARER À L'ASSUREUR\n"
            "5.1 Déclaration initiale du risque\n"
            "Le preneur, et la personne assurée si l'assureur le demande, doivent déclarer à l'assureur "
            "les circonstances qu'ils connaissent et qui sont de nature à influencer de façon importante un "
            "assureur dans :\n"
            "\t son analyse du risque;\n"
            "\t sa décision d'accepter ou non le risque; ou\n"
            "\t l'établissement de la prime d'assurance.\n"
            "Par contre, le preneur et la personne assurée n'ont pas à déclarer les circonstances que "
            "l'assureur connaît déjà ou qu'il est présumé connaître en raison de leur notoriété, sauf si "
            "l'assureur pose des questions à ce sujet.\n"
            "5.2 Aggravation du risque\n"
            "5.2.1 Obligation de la personne assurée\n"
            "La personne assurée doit déclarer sans tarder à l'assureur les circonstances qui "
            "aggravent les risques spécifiés dans le contrat d'assurance.\n"
            "Les circonstances à déclarer doivent résulter des faits et gestes de la personne "
            "assurée. Elles doivent aussi être de nature à influencer de façon importante un assureur dans :\n"
            "\t son analyse du risque;\n"
            "\t sa décision de maintenir le contrat d'assurance ou d'y mettre fin; ou\n"
            "\t l'établissement de la prime d'assurance.\n"
            "5.2.2 Droits de l'assureur\n"
            "L'assureur qui est informé de nouvelles circonstances peut :\n"
            "\t proposer à l'assuré désigné, par écrit, une nouvelle prime d'assurance.\n"
            "L'assuré désigné doit alors accepter et payer la nouvelle prime d'assurance "
            "dans les 30 jours de la proposition qui lui est faite. S'il ne le fait pas, le contrat "
            "d'assurance prend fin; ou\n"
            "\t résilier le contrat d'assurance en respectant les conditions énoncées à l'article 3 "
            "de la section « Prise d'effet, renouvellement et expiration du contrat d'assurance ».\n"
            "Par contre, si l'assureur continue d'accepter le paiement de la prime d'assurance ou "
            "s'il paie une indemnité après un sinistre, il est réputé avoir accepté les nouvelles "
            "circonstances qui lui ont été déclarées.\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "5.3 Conséquences en cas de fausses déclarations ou d'informations non déclarées\n"
            "5.3.1 Conséquences d'application au chapitre A\n"
            "A. Annulation du chapitre A\n"
            "À tout moment, l'assureur peut demander l'annulation du chapitre A si :\n"
            "a) le preneur ou une personne assurée :\n"
            "\t a fait une fausse déclaration sur les informations à déclarer aux "
            "paragraphes 5.1 et 5.2 de la présente section; ou\n"
            "\t n'a volontairement pas déclaré une information visée aux "
            "paragraphes 5.1 et 5.2 de la présente section (appelée une « réticence »);\n"
            "et\n"
            "b) cette fausse déclaration ou cette réticence est de nature à influencer sensiblement un assureur "
            "raisonnable dans sa décision d'accepter le risque.\n"
            "B. Réduction de l'indemnité à la suite d'un sinistre\n"
            "Si, à la suite d'un sinistre, l'assureur ne réussit pas à démontrer que la fausse déclaration ou la "
            "réticence était de nature à influencer sensiblement un assureur raisonnable dans sa décision d'accepter "
            "le risque, il devra payer une partie de l'indemnité.\n"
            "L'indemnité est calculée en proportion de la prime d'assurance établie par l'assureur avant qu'il prenne "
            "connaissance de la fausse déclaration ou de la réticence, divisée par la prime d'assurance qu'il aurait "
            "fixée si le preneur ou la personne assurée lui avait fourni les informations qu'il devait déclarer.\n"
            "5.3.2 Conséquences d'application au chapitre B\n"
            "A. Annulation du chapitre B\n"
            "À tout moment, l'assureur peut demander l'annulation du chapitre B si :\n"
            "a) le preneur ou une personne assurée :\n"
            "\t a fait une fausse déclaration sur les informations à déclarer aux paragraphes 5.1 et 5.2 de la "
            "présente section; ou\n"
            "\t n'a volontairement pas déclaré une information visée aux paragraphes 5.1 et 5.2 de la présente "
            "section (appelée une « réticence »);\n"
            "et\n"
            "b) cette fausse déclaration ou cette réticence est de nature à influencer sensiblement un assureur "
            "raisonnable dans sa décision d'accepter le risque.\n"
            "À la suite d'un sinistre, l'assureur peut demander l'annulation du chapitre B même si le sinistre "
            "découle d'un risque qui n'a pas été dénaturé par la fausse déclaration ou la réticence.\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "B. Réduction de l'indemnité à la suite d'un sinistre\n"
            "À la suite d'un sinistre, malgré toute fausse déclaration ou réticence, l'assureur devra payer une "
            "partie de l'indemnité s'il ne réussit pas à démontrer :\n"
            "\t la mauvaise foi de l'assuré désigné ou du preneur; ou\n"
            "\t qu'il n'aurait pas accepté le risque s'il avait été informé des vraies circonstances.\n"
            "L'indemnité est calculée en proportion de la prime d'assurance établie par l'assureur avant qu'il prenne "
            "connaissance de la fausse déclaration ou de la réticence, divisée par la prime d'assurance qu'il aurait "
            "fixée si le preneur ou la personne assurée lui avait fourni les informations qu'il devait déclarer.\n"
            "6. NON-RESPECT D'UN ENGAGEMENT FORMEL\n"
            "En cas de non-respect d'un engagement formel qui aggrave le risque, la garantie qui couvre le risque visé "
            "par l'engagement formel est suspendue.\n"
            "La suspension prend fin lorsque :\n"
            "\t une personne assurée respecte de nouveau son engagement formel; ou\n"
            "\t l'assureur donne son consentement.\n"
            "Les engagements formels peuvent, entre autres, se retrouver :\n"
            "\t à la section « Conditions particulières »;\n"
            "\t dans les avenants.\n"
            "7. USAGES INTERDITS D'UN VÉHICULE ASSURÉ\n"
            "Les personnes assurées ne doivent pas conduire le véhicule assuré, ni le faire fonctionner, dans les "
            "situations suivantes :\n"
            "a) Les personnes assurées :\n"
            "\t n'ont pas au moins 16 ans ou l'âge légal pour conduire; et\n"
            "\t ne sont pas autorisées à conduire selon la loi, ou aptes à conduire ou à faire fonctionner le "
            "véhicule.\n"
            "b) Pour faire du transport ou du commerce illégalement.\n"
            "c)Pour participer à une course ou à une épreuve de vitesse.\n"
            "Les personnes assurées ne doivent pas non plus permettre à toute autre personne de faire usage du "
            "véhicule assuré dans ces situations.\n"
            "8. EXAMEN DES VÉHICULES ASSURÉS\n"
            "À tout moment raisonnable, l'assureur a le droit d'examiner les véhicules assurés ou leurs équipements "
            "et leurs accessoires.\n"
            "9. ENVOI DES AVIS PAR L'ASSUREUR ET L'ASSURÉ DÉSIGNÉ\n"
            "Les avis destinés à l'assureur peuvent être envoyés à l'assureur, ou à son représentant autorisé, "
            "par tout moyen de communication reconnu.\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "Les avis destinés à l'assuré désigné peuvent lui être remis en mains propres ou lui être envoyés "
            "par courrier à sa dernière adresse connue.\n"
        )

        self.assertEqual(expected, actual)

    def test_generation_loss_and_claims_reporting(self):
        actual = generate_loss_and_claims_reporting_text()
        expected = (
            "DÉCLARER UN SINISTRE ET FAIRE UNE RÉCLAMATION\n"
            "1. QUOI FAIRE LORS D'UN SINISTRE\n"
            "1.1 Déclarer certaines informations à l'assureur\n"
            "1.1.1 Déclarer le sinistre\n"
            "Dès qu'une personne assurée a connaissance d'un sinistre qui pourrait être couvert par le contrat "
            "d'assurance, elle doit en informer l'assureur.\n"
            "Toutes les personnes intéressées peuvent aussi en informer l'assureur.\n"
            "Si cette obligation de déclarer le sinistre n'est pas respectée et que l'assureur en subit un "
            "préjudice, la personne assurée perd son droit à l'indemnisation.\n"
            "1.1.2 Déclarer certaines autres informations\n"
            "Lorsque l'assureur le demande, la personne assurée doit l'informer le plus tôt possible de toutes "
            "les circonstances relatives au sinistre, y compris :\n"
            "\t la cause probable du sinistre;\n"
            "\t la nature et l'étendue des dommages;\n"
            "\t l'endroit où se trouve le véhicule assuré ou tout autre bien;\n"
            "\t les droits de toute autre personne;\n"
            "\t les autres contrats d'assurance qui peuvent s'appliquer.\n"
            "La personne assurée doit aussi remettre à l'assureur les pièces justificatives qui permettent de "
            "prouver ces informations. Elle doit affirmer sous serment que toutes les informations fournies sont "
            "véridiques.\n"
            "Si, pour un motif sérieux, la personne assurée ne peut pas respecter ces obligations le plus tôt "
            "possible, elle a droit à un délai raisonnable pour le faire.\n"
            "Si la personne assurée ne respecte pas ces obligations, toute personne intéressée peut le faire "
            "à sa place.\n"
            "La personne assurée doit aussi remettre à l'assureur, dans les meilleurs délais, une copie de tous "
            "les documents qu'elle reçoit concernant une réclamation, y compris :\n"
            "\t les avis;\n"
            "\t les lettres;\n"
            "les assignations et tout autre acte de procédure.\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "1.1.3 Conséquences en cas de déclarations mensongères\n"
            "La personne qui fait une déclaration mensongère relative au sinistre perd son droit à l'indemnisation."
            "Elle perd ce droit uniquement pour les dommages causés par la réalisation du risque auquel se rattache "
            "la déclaration mensongère.\n"
            "Par contre, si la réalisation de ce risque a causé des dommages tant à des biens à usage professionnel "
            "qu'à des biens à usage personnel, cette personne perd son droit à l'indemnisation uniquement pour "
            "les dommages causés à la catégorie de biens à laquelle se rattache la déclaration mensongère.\n"
            "1.2 Respecter certaines obligations relatives au véhicule assuré\n"
            "1.2.1 Ne pas abandonner le véhicule assuré\n"
            "La personne assurée ne doit pas abandonner le véhicule assuré ou tout autre bien endommagé, sans le "
            "consentement de l'assureur.\n"
            "1.2.2 Faciliter le sauvetage du véhicule assuré et les vérifications de l'assureur\n"
            "La personne assurée doit faciliter le sauvetage du véhicule assuré et de tout autre bien assuré.\n"
            "Elle doit aussi faciliter les vérifications de l'assureur. Elle doit, entre autres, permettre à "
            "l'assureur et à ses représentants de visiter tout lieu et d'examiner le véhicule assuré ou "
            "ses équipements et accessoires.\n"
            "1.2.3 Protéger le véhicule assuré\n"
            "La personne assurée doit se charger de protéger le véhicule assuré contre tout danger de perte "
            "ou de dommage supplémentaire, dans la mesure du possible et aux frais de l'assureur.\n"
            "Si elle ne respecte pas cette obligation, tout dommage qui en découle, dans quelque mesure que "
            "ce soit, sera à ses frais.\n"
            "1.2.4 Ne pas réparer le véhicule assuré ni enlever des éléments\n"
            "Tant que l'assureur n'a pas examiné le véhicule assuré dans un délai raisonnable, comme prévu à "
            "l'article 8 de la section « Conditions générales » :\n"
            "\t aucune réparation ne doit être faite au véhicule; et\n"
            "\t aucun élément utile à l'évaluation des dommages ne doit être enlevé du véhicule.\n"
            "Par contre, ces actions peuvent être posées dans l'une des situations suivantes :\n"
            "\t si elles sont nécessaires pour protéger le véhicule assuré;\n"
            "\t si l'assureur donne son consentement par écrit.\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "1.3 Ne pas se prononcer sur sa responsabilité et ne pas régler la réclamation\n"
            "Sauf à ses propres frais, la personne assurée, à la suite d'un sinistre :\n"
            "\t ne doit pas se prononcer sur sa responsabilité; et\n"
            "\t ne doit pas régler ou tenter de régler une réclamation.\n"
            "Si une personne assurée conclut une entente par rapport au sinistre (appelée une « transaction ») "
            "et ce, sans le consentement de l'assureur, l'assureur n'est pas lié par cette entente.\n"
            "1.4 Collaborer avec l'assureur\n"
            "La personne assurée doit collaborer avec l'assureur dans le traitement de toute réclamation.\n"
            "2. COMMENT SE DÉTERMINE LA VALEUR DES DOMMAGES\n"
            "La valeur des dommages payable par l'assureur ne peut pas dépasser la « valeur au jour du sinistre » "
            "du véhicule assuré.\n"
            "Lorsque l'assuré désigné a été indemnisé, l'assureur a le droit de récupérer le bien ou la partie du "
            "bien endommagé.\n"
            "2.1 Valeur des dommages payable par l'assureur pour la réparation du véhicule assuré\n"
            "La valeur des dommages est déterminée, entre autres, selon le prix des matériaux au jour du sinistre.\n"
            "Les matériaux utilisés pour la réparation du véhicule assuré, ou pour le remplacement de pièces "
            "endommagées, doivent être de même nature et de même qualité, en tenant compte de la dépréciation de "
            "quelque nature qu'elle soit.\n"
            "2.1.1 Précisions pour les pièces de carrosserie\n"
            "Dans les cas suivants, l'assureur détermine le prix des matériaux sur la base des pièces d'origine "
            "du fabricant :\n"
            "\t le véhicule est âgé de moins de 2 ans ou il a moins de 40 000 km;\n"
            "\t le véhicule est un véhicule à usage commercial âgé de moins d'un an.\n"
            "Dans les autres cas, l'assureur peut se baser sur le prix de pièces similaires de carrosserie."
            "Par contre, l'assuré désigné peut demander des pièces d'origine du fabricant si elles sont disponibles."
            "Il doit en informer l'assureur au moment où il déclare le sinistre. L'assureur précisera alors les "
            "conditions applicables et les coûts supplémentaires que l'assuré désigné devra payer.\n"
            "2.1.2 Pièces non disponibles ou qui ne sont plus fabriquées\n"
            "Si parmi les matériaux nécessaires à la réparation du véhicule, certaines pièces de rechange ne sont "
            "pas disponibles ou ne sont plus fabriquées, l'assureur n'est tenu qu'au dernier prix courant des "
            "pièces d'origine du fabricant.\n"
            "L'assureur peut aussi tenir compte de la dépréciation de quelque nature qu'elle soit.\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "2.2 Valeur des dommages payable par l'assureur pour le véhicule assuré qui est une perte totale\n"
            "Lorsque le véhicule assuré est une perte totale ou que la perte est réputée totale, la valeur des "
            "dommages équivaut à la « valeur au jour du sinistre » du véhicule assuré.\n"
            "Malgré la perte totale ou réputée totale, l'assuré désigné peut demander que le véhicule soit "
            "remis dans l'état où il était au jour du sinistre.\n"
            "L'assuré désigné doit remettre à l'assureur les pièces justificatives qui permettent de démontrer "
            "l'état de son véhicule au jour du sinistre.\n"
            "Dans un tel cas, l'assureur doit accepter cette demande s'il juge que les coûts sont raisonnables "
            "pour remettre le véhicule dans l'état où il était au jour du sinistre.\n"
            "2.3 Cas où l'assureur peut décider de faire réparer, reconstruire ou remplacer le bien endommagé\n"
            "S'il n'y a pas d'arbitrage, et sous réserve des droits des créanciers prioritaires et hypothécaires, "
            "l'assureur peut décider de réparer, reconstruire ou remplacer le bien endommagé avec d'autres "
            "biens de même nature et de même qualité, et ce, au lieu de payer une indemnité en argent.\n"
            "Avant de procéder à la réparation, à la reconstruction ou au remplacement du bien, l'assureur "
            "doit en informer l'assuré désigné par écrit dans les 7 jours où il reçoit la demande d'indemnité.\n"
            "De plus, la réparation, la reconstruction ou le remplacement doit être fait dans un délai raisonnable.\n"
            "3. DROIT DE L'ASSUREUR APRÈS AVOIR PAYÉ UNE INDEMNITÉ (DROIT DE SUBROGATION)\n"
            "3.1 Règle générale\n"
            "Après avoir payé une indemnité, l'assureur est subrogé dans les droits de la personne assurée "
            "contre la personne responsable des dommages. Cela signifie que les droits de la personne assurée "
            "sont transférés à l'assureur.\n"
            "Cette subrogation s'opère jusqu'à concurrence de l'indemnité que l'assureur a payée.\n"
            "Si l'assureur ne peut pas exercer son droit de subrogation du fait de la personne assurée, "
            "l'assureur peut être libéré de ses obligations envers la personne assurée, en partie ou en totalité.\n"
            "3.2 Exceptions\n"
            "Dans les deux cas suivants, l'assureur ne peut pas demander à la personne responsable des dommages "
            "de lui rembourser l'indemnité qu'il a payée :\n"
            "a) Lorsque cette personne fait partie de la maison de la personne assurée.\n"
            "b) Pour le chapitre B seulement, lorsque cette personne avait un pouvoir de direction ou de "
            "gestion sur le véhicule assuré ou en avait la garde, avec le consentement de l'assuré désigné."
            "Cette dernière exception ne s'applique pas si cette personne :\n"
            "\t exerçait une activité professionnelle de garagiste au moment du sinistre; ou\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "\t n'a pas respecté le contrat d'assurance.\n"
            "4. ARBITRAGE EN CAS DE DÉSACCORD ENTRE L'ASSURÉ DÉSIGNÉ ET L'ASSUREUR 26\n"
            "4.1 Faire une demande d'arbitrage\n"
            "L'assuré désigné ou l'assureur peuvent faire une demande d'arbitrage s'ils ne sont pas d'accord "
            "sur les questions suivantes :\n"
            "\t la nature, l'étendue ou la valeur des dommages;\n"
            "\t si la réparation ou le remplacement est suffisant.\n"
            "Cette demande d'arbitrage peut être faite même si la validité du contrat d'assurance est contestée.\n"
            "4.1.1 Demande faite par l'assuré désigné\n"
            "L'assuré désigné doit envoyer un avis écrit à l'assureur et y préciser la raison du désaccord.\n"
            "L'assureur doit accepter la demande d'arbitrage faite par l'assuré désigné et lui envoyer un "
            "accusé de réception dans les 15 jours francs où il reçoit l'avis.\n"
            "4.1.2 Demande faite par l'assureur\n"
            "L'assureur doit envoyer un avis écrit à l'assuré désigné qui précise la raison du désaccord.\n"
            "L'assuré désigné doit confirmer à l'assureur son acceptation ou son refus de soumettre le désaccord à "
            "l'arbitrage dans les 15 jours francs où il reçoit l'avis.\n"
            "4.2 Choisir les experts et l'arbitre\n"
            "L'assureur et l'assuré désigné doivent chacun choisir un expert.\n"
            "Selon la nature du désaccord, les deux experts choisis doivent :\n"
            "\t déterminer la nature, l'étendue et la valeur des dommages. Pour ce faire, ils doivent évaluer "
            "séparément la « valeur au jour du sinistre » et le coût de réparation ou de remplacement; ou\n"
            "\t évaluer si la réparation ou le remplacement est suffisant.\n"
            "Si les évaluations des experts sont différentes, ils doivent tenter de s'entendre sur une valeur "
            "commune.\n"
            "S'ils n'y arrivent pas, ils doivent soumettre leur différend à un arbitre neutre qu'ils choisissent, "
            "c'est-à-dire un arbitre qui ne représente ni les intérêts de l'assureur ni les intérêts de l'assuré "
            "désigné.\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "Dans les cas suivants, l'assureur ou l'assuré désigné doit demander à un tribunal compétent à "
            "l'endroit de l'arbitrage de nommer les experts ou l'arbitre :\n"
            "\t l'assureur ou l'assuré désigné n'a pas choisi son expert dans les 30 jours francs de la "
            "date de l'avis;\n"
            "\t les experts n'ont pas choisi un arbitre dans les 15 jours francs de leur nomination; l'un "
            "des experts ou l'arbitre refuse de faire l'arbitrage ou n'est pas disponible.\n"
            "4.3 Valeur des dommages payable par l'assureur\n"
            "Même s'il y a un arbitrage, l'assureur doit payer la partie de la valeur des dommages qui n'est pas "
            "contestée. Ce paiement doit être fait au plus tard :\n"
            "\t dans les 60 jours où le sinistre a été déclaré; ou\n"
            "\t dans les 60 jours où l'assureur a reçu les informations ou les pièces justificatives qu'il a exigées.\n"
            "Par contre, si la validité ou l'application du contrat d'assurance est contestée, l'assureur n'a "
            "pas à payer ce montant dans ces délais.\n"
            "À la suite de l'arbitrage, l'assureur doit payer le montant fixé par l'arbitre dans les 15 jours à "
            "compter du moment où l'assuré désigné a accepté la décision de l'arbitre.\n"
            "4.4 Déroulement de l'arbitrage\n"
            "L'arbitrage doit se dérouler selon les articles 940 à 951.2 du Code de procédure civile du Québec, en "
            "tenant compte des adaptations nécessaires en raison des règles particulières prévues au contrat "
            "d'assurance.\n"
            "Comme le prévoit l'article 944.1 du Code de procédure civile du Québec, l'arbitre peut décider "
            "quelle procédure il appliquera lors de l'arbitrage. Il doit malgré tout s'assurer de respecter les "
            "règles prévues aux articles 940 à 951.2 du Code de procédure civile du Québec.\n"
            "4.5 Choix de la langue\n"
            "L'arbitre, l'assureur et l'assuré désigné peuvent utiliser la langue de leur choix pendant "
            "l'arbitrage. Des mesures doivent être prises pour assurer la compréhension de tous.\n"
            "4.6 Endroit où se déroule l'arbitrage\n"
            "Le lieu de l'arbitrage est déterminé en fonction du domicile de l'assuré désigné.\n"
            "4.7 Décision de l'arbitre\n"
            "L'arbitre rend une décision en se basant sur les lois applicables au Québec.\n"
            "Sa décision doit être écrite et motivée. Elle doit aussi être signée et inclure la date et le lieu où "
            "elle a été rendue.\n"
            "La décision doit être envoyée à l'assureur et à l'assuré désigné dans les 30 jours de la date "
            "où elle a été rendue.\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "4.8 Frais et honoraires de l'arbitrage\n"
            "L'assureur et l'assuré désigné paient les frais et les honoraires de leur propre expert et la "
            "moitié des frais et honoraires de l'arbitrage.\n"
            "Par contre, si l'arbitre considère que la façon de partager les frais et les honoraires de "
            "l'arbitrage n'est pas justifiée ou équitable dans les circonstances, il peut en décider autrement.\n"
            "5. MAINTIEN DES DROITS DE L'ASSURÉ DÉSIGNÉ ET DE L'ASSUREUR\n"
            "Les actes posés par l'assuré désigné ou l'assureur ne sont pas considérés comme une renonciation "
            "aux droits que leur donne le contrat d'assurance s'ils sont posés dans le cadre:\n"
            "\t d'une enquête sur un sinistre;\n"
            "\t d'un règlement de sinistre;\n"
            "\t d'un arbitrage;\n"
            "\t d'une demande d'indemnité.\n"
            "6. DÉLAI POUR ENTREPRENDRE UNE ACTION DÉCOULANT DU CONTRAT D'ASSURANCE (DÉLAI DE PRESCRIPTION)\n"
            "Une action qui découle du contrat d'assurance, incluant toute action en justice, doit être exercée "
            "dans les 3 ans à partir du moment où le droit d'action a commencé à exister.\n"
        )

        self.assertEqual(expected, actual)

    def test_generation_dates(self):
        actual = generate_dates_text()
        expected = (
            "PRISE D'EFFET, RENOUVELLEMENT ET EXPIRATION DU CONTRAT D'ASSURANCE\n"
            "1. PRISE D'EFFET ET EXPIRATION DU CONTRAT D'ASSURANCE\n"
            "Le contrat d'assurance prend effet et expire aux dates et à l'heure écrites à l'article 2 de la section "
            "« Conditions particulières » ou, selon le cas, dans les avenants.\n"
            "2. RENOUVELLEMENT DU CONTRAT D'ASSURANCE\n"
            "À sa date d'expiration, le contrat d'assurance se renouvelle automatiquement, à moins d'un avis "
            "contraire de l'assuré désigné ou de l'assureur.\n"
            "Il se renouvelle pour la même prime d'assurance et la même durée, à moins d'un avis contraire de "
            "l'assuré désigné ou de l'assureur.\n"
            "L'avis envoyé par l'assureur peut être un avis de non-renouvellement ou un avis pour modifier la "
            "prime d'assurance. Cet avis doit être adressé à l'assuré désigné au plus tard 30 jours avant la "
            "date d'expiration du contrat d'assurance, à sa dernière adresse connue.\n"
            "Si l'assuré désigné utilise les services d'un courtier en assurance, l'avis de l'assureur doit être "
            "remis au courtier en assurance, qui doit ensuite le remettre à l'assuré désigné.\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "3. RÉSILIATION DU CONTRAT D'ASSURANCE (METTRE FIN AU CONTRAT D'ASSURANCE)\n"
            "3.1 Résiliation par l'assuré désigné\n"
            "3.1.1. Conditions à respecter\n"
            "À tout moment, l'assuré désigné peut résilier le contrat d'assurance en envoyant un "
            "avis écrit à l'assureur.\n"
            "Les assurés désignés peuvent mandater un ou plusieurs d'entre eux pour envoyer un avis en leur nom "
            "à tous.\n"
            "La résiliation prend effet dès que l'assureur reçoit l'avis de chacun des assurés désignés "
            "ou de leur mandataire.\n"
            "3.1.2 Remboursement de la prime d'assurance\n"
            "Si le contrat d'assurance est résilié par l'assuré désigné, l'assureur doit lui rembourser la "
            "partie de la prime d'assurance payée en trop, telle que calculée selon le « Tableau de résiliation ».\n"
            "Ce « Tableau de résiliation » fait partie du contrat d'assurance.\n"
            "Par contre, si la prime d'assurance a été payée à l'assureur par le courtier en assurance, "
            "l'assuré désigné peut être remboursé uniquement pour ce qu'il a effectivement payé ou remboursé au "
            "courtier en assurance.\n"
            "3.2 Résiliation par l'assureur\n"
            "3.2.1 Conditions à respecter\n"
            "A. Résiliation faite dans les 60 jours\n"
            "L'assureur peut résilier le contrat d'assurance dans les 60 jours de la prise d'effet du contrat "
            "d'assurance.\n"
            "Il doit envoyer un avis écrit à chacun des assurés désignés ou à leur mandataire.\n"
            "La résiliation prend effet 15 jours après la réception de l'avis par chacun des assurés désignés ou "
            "par leur mandataire, à leur dernière adresse connue.\n"
            "B. Résiliation faite après 60 jours\n"
            "Plus de 60 jours après la prise d'effet du contrat d'assurance, l'assureur peut résilier le "
            "contrat d'assurance uniquement dans l'une des situations suivantes :\n"
            "\t il y a eu une aggravation des risques de nature à influencer sensiblement un assureur raisonnable "
            "dans sa décision de maintenir le contrat d'assurance;\n"
            "\t la prime d'assurance n'a pas été payée.\n"
            "L'assureur doit alors envoyer un avis écrit à chacun des assurés désignés ou à leur mandataire.\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "La résiliation prend effet, selon le cas :\n"
            "\t 30 jours après la réception de l'avis par chacun des assurés désignés ou par leur mandataire, "
            "à leur dernière adresse connue;\n"
            "\t 15 jours après la réception de l'avis par chacun des assurés désignés ou par leur mandataire, "
            "à leur dernière adresse connue, si le véhicule désigné est un véhicule visé par le titre VIII.I "
            "du Code de la sécurité routière et qu'il n'est pas un autobus scolaire.\n"
            "3.2.2 Remboursement de la prime d'assurance\n"
            "Si l'assureur résilie le contrat d'assurance, il n'a droit qu'à la partie de la prime d'assurance "
            "équivalente au nombre de jours pendant lesquels l'assuré désigné a effectivement bénéficié du "
            "contrat d'assurance.\n"
            "Si l'assuré désigné a payé la prime d'assurance à l'avance, l'assureur doit lui rembourser ce qui "
            "a été payé en trop. Par contre, si la prime d'assurance a été payée à l'assureur par le courtier "
            "en assurance, l'assuré désigné peut être remboursé uniquement pour ce qu'il a effectivement payé "
            "ou remboursé au courtier en assurance.\n"
            f"{NEW_PAGE_TAG}\n"
        )

        self.assertEqual(expected, actual)

    def test_generation_cancellation_table(self):
        actual = generate_cancellation_table_text()
        expected = (
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "TABLEAU DE RÉSILIATION\n"
            f"{NEW_PAGE_TAG}\n"
        )

        self.assertEqual(expected, actual)

    def test_generation_definition(self):
        actual = generate_definitions_text()
        expected = (
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "DÉFINITIONS\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "Sauf si le contexte indique un sens différent, les définitions ci-dessous s'appliquent aux mots et aux "
            "expressions en caractère gras dans le contrat d'assurance. À noter qu'en raison du contexte, un mot "
            "défini ou une expression définie peut ne pas être en caractère gras dans le contrat d'assurance; dans un "
            "tel cas, ils sont utilisés dans leur sens ordinaire et la définition ne s'y applique pas.\n"
            "Certaines des définitions sont une version simplifiée de celles déjà prévues dans les lois suivantes :\n"
            "\t le Code civil du Québec;\n"
            "\t le Code de procédure civile du Québec;\n"
            "\t la Loi sur l'assurance automobile; et\n"
            "\t la Loi sur les véhicules hors route.\n"
            "En cas d'ambiguïté ou de divergence, c'est le texte de ces lois qui a priorité.\n"
            "ACTIVITÉ PROFESSIONNELLE DE GARAGISTE : entre autres, toute activité professionnelle relative à la "
            "garde, la vente, l'équipement, la réparation, l'entretien et le remisage de véhicules automobiles, "
            "ainsi qu'au stationnement, au déplacement et au contrôle du bon fonctionnement de ces véhicules.\n"
            "ASSURÉ DÉSIGNÉ : toute personne nommée à l'article 1 de la section « Conditions particulières ».\n"
            "ASSUREUR : l'assureur du présent contrat d'assurance.\n"
            "AUTRE PERSONNE : toute personne qui n'est pas une « personne assurée » au contrat d'assurance.\n"
            "AVENANT : document qui modifie le contrat d'assurance. Il est officiellement appelé « Formulaire "
            "d'avenant du Québec » ou « F.A.Q. ».\n"
            "CONJOINT : celui ou celle qui, au moment du sinistre :\n"
            "\t a) est marié et cohabite avec la personne à qui il est marié;\n"
            "\t b) n'est pas marié, mais vit maritalement et cohabite avec une personne de sexe différent ou "
            "de même sexe. Cette personne doit aussi être publiquement représentée comme son conjoint. Ces "
            "conditions doivent exister depuis au moins trois ans ou, dans les cas suivants, depuis au moins un an :\n"
            "\t un enfant est né ou est à naître de leur union;\n"
            "\t ils ont adopté ensemble un enfant;\n"
            "\t l'un a adopté un enfant de l'autre.\n"
            "DOMMAGE :\n"
            "\t Au chapitre A, « dommage » fait référence aux dommages matériels et aux dommages corporels.\n"
            "\t Au chapitre B, « dommage » fait référence seulement aux dommages matériels.\n"
            "DOMMAGE CORPOREL : tout dommage de nature physique ou psychique, y compris le décès.\n"
            "DOMMAGE MATÉRIEL : tout dommage causé à un véhicule automobile ou à un autre bien, y compris "
            "leur disparition.\n"
            "FRANCHISE : montant laissé à la charge de l'assuré désigné.\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "MONTANT D'ASSURANCE : montant maximum payable par l'assureur, écrit à l'article 4 de la "
            "section « Conditions particulières » ou dans un avenant.\n"
            "PRENEUR : personne qui soumet une demande à l'assureur pour conclure un contrat d'assurance. "
            "Cette demande (appelée une « proposition d'assurance ») peut être faite pour le preneur lui-même ou "
            "pour une autre personne. Le preneur n'est pas nécessairement l'assuré désigné.\n"
            "PRIME D'ASSURANCE : montant payable à l'assureur en échange des garanties accordées par le contrat "
            "d'assurance.\n"
            "PROPRIÉTAIRE : personne qui acquiert un véhicule automobile ou qui le possède en vertu de l'un "
            "des documents suivants :\n"
            "\t un document qui confirme son statut de propriétaire du véhicule (appelé un « titre de propriété »);\n"
            "\t un document qui lui donne le droit de devenir propriétaire du véhicule à certaines conditions ou "
            "à un certain moment;\n"
            "\t un document qui lui donne le droit de bénéficier du véhicule comme s'il en était propriétaire, "
            "pendant un certain temps seulement;\n"
            "\t un contrat de location d'une durée d'au moins un an.\n"
            "REMORQUE OU SEMI-REMORQUE DONT L'ASSURÉ DÉSIGNÉ EST PROPRIÉTAIRE : toute remorque ou toute "
            "semi-remorque dont l'assuré désigné est propriétaire et qui n'est pas désignée à l'article 3 de "
            "la section « Conditions particulières », si :\n"
            "\t a) la remorque ou la semi-remorque n'est pas conçue ni utilisée pour le transport de personnes, ni "
            "à des fins de démonstration, de vente, de bureau ou d'habitation; et\n"
            "\t b) la remorque ou la semi-remorque est attelée à un véhicule automobile utilisé à des "
            "fins personnelles qui est assuré, ou n'est pas attelée à un tel véhicule à la condition qu'elle "
            "le soit habituellement.\n"
            "RISQUE NUCLÉAIRE : risque découlant de la nature dangereuse de substances :\n"
            "\t qui ont des propriétés radioactives, toxiques ou explosives; et\n"
            "\t qui sont désignées dans la Loi sur la sûreté et la réglementation nucléaire ou ses règlements.\n"
            "SINISTRE : un risque qui se réalise et qui cause un dommage.\n"
            "VÉHICULE AUTOMOBILE : tout véhicule qui est mis en mouvement par un pouvoir autre que la "
            "force musculaire et qui est adapté au transport sur les chemins publics, mais non sur les rails.\n"
            "VÉHICULE AUTOMOBILE UTILISÉ À DES FINS PERSONNELLES : inclut, entre autres, tout véhicule "
            "automobile de type utilitaire lorsqu'il est utilisé à des fins personnelles et que son poids total "
            "en charge ne dépasse pas 4500 kg (10000 lb).\n"
            "VÉHICULE DE REMPLACEMENT TEMPORAIRE : tout véhicule automobile utilisé de façon temporaire en "
            "remplacement d'un véhicule désigné, si les conditions suivantes sont respectées :\n"
            "\t le véhicule désigné est inutilisable pour l'une des raisons suivantes : panne, réparation, "
            "entretien, perte, destruction, vente ou contrôle du bon fonctionnement; et\n"
            "\t ni l'assuré désigné ni toute personne qui a le même domicile que lui n'est propriétaire du "
            "véhicule de remplacement temporaire.\n"
            "VÉHICULE DÉSIGNÉ : tout véhicule automobile, toute remorque ou toute semi-remorque décrit à "
            "l'article 3 de la section « Conditions particulières » ou inclus dans une désignation générale de "
            "cet article.\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "VÉHICULE DONT L'ASSURÉ DÉSIGNÉ EST NOUVELLEMENT PROPRIÉTAIRE : tout véhicule automobile, toute "
            "remorque ou toute semi-remorque dont l'assuré désigné est nouvellement propriétaire, si les "
            "conditions suivantes sont respectées :\n"
            "\t a) Dans les 14 jours de la prise de possession du véhicule, l'assuré désigné informe "
            "l'assureur qu'il en est le propriétaire.\n"
            "\t b) Le véhicule remplace ou s'ajoute à un ou plusieurs véhicules désignés. Si le véhicule "
            "s'ajoute à un ou plusieurs véhicules désignés :\n"
            "\t l'assuré désigné ne doit posséder aucune autre assurance spécifique pour ce véhicule à "
            "la date du sinistre; et\n"
            "\t l'assureur doit assurer tous les autres véhicules dont l'assuré désigné est propriétaire "
            "à la date de la prise de possession du véhicule.\n"
            "\t c) L'assuré désigné s'engage à payer la prime d'assurance additionnelle exigée par l'assureur.\n"
            "Si l'assuré désigné exerce une activité professionnelle de vente de véhicules automobiles, "
            "les véhicules automobiles qu'il acquiert aux fins de son activité ne sont pas considérés comme "
            "des « véhicules dont l'assuré désigné est nouvellement propriétaire ».\n"
            "VÉHICULE DONT L'ASSURÉ DÉSIGNÉ N'EST PAS PROPRIÉTAIRE : tout véhicule automobile dont l'assuré "
            "désigné n'est pas propriétaire, si les conditions énoncées ci-dessous sont respectées. Ces conditions "
            "sont différentes si l'assuré désigné est un individu ou une personne morale, une société ou une "
            "association.\n"
            "a) Si l'assuré désigné est un individu : au moment du sinistre, le conducteur doit être l'assuré "
            "désigné ou son conjoint.\n"
            "Par contre, dans les quatre cas d'exception énumérés ci-dessous, le véhicule est considéré "
            "comme un véhicule assuré seulement s'il est conduit par le propriétaire d'un véhicule désigné.\n"
            "De plus, seul le propriétaire sera couvert, et ce, pour un montant d'assurance réduit (voir les "
            "paragraphes 6.7 et 6.8 du chapitre A) :\n"
            "\t 1. Au moment du sinistre, le véhicule est conduit dans le cadre d'une activité professionnelle de "
            "garagiste.\n"
            "\t 2. Le propriétaire du véhicule est une personne qui a le même domicile que l'assuré désigné.\n"
            "\t 3. Le véhicule est fourni par l'employeur de l'assuré désigné ou par l'employeur d'une "
            "personne qui a le même domicile que lui.\n"
            "\t 4. Le véhicule est affecté, à l'extérieur du Québec :\n"
            "- à l'usage de taxi, d'autobus, d'autocar; ou\n"
            "- à la livraison commerciale.\n"
            "b) Si l'assuré désigné est une personne morale, une société ou une association :\n"
            "1. Au moment du sinistre, le véhicule doit être conduit par l'un des conducteurs suivants ou "
            "par leur conjoint :\n"
            "\t un employé de l'assuré désigné;\n"
            "\t un actionnaire de l'assuré désigné;\n"
            "\t un membre de l'assuré désigné;\n"
            "\t un associé de l'assuré désigné.\n"
            "2. Ces conducteurs doivent habituellement faire usage du véhicule désigné.\n"
            f"{NEW_PAGE_TAG}\n"
            "FORMULAIRE DES PROPRIÉTAIRES\n"
            "F.P.Q. N 1 - POLICE D'ASSURANCE AUTOMOBILE DU QUÉBEC (2014-03)\n"
            "Approuvée par l'Autorité des marchés financiers\n"
            "3. Ces conducteurs ou leur conjoint ne doivent pas être propriétaires d'un véhicule automobile.\n"
            "4. Au moment du sinistre, le véhicule n'est pas conduit dans le cadre d'une activité "
            "professionnelle de garagiste.\n"
            "5. Le véhicule ne doit pas être affecté, à l'extérieur du Québec :\n"
            "\t à l'usage de taxi, d'autobus, d'autocar; ou\n"
            "\t à la livraison commerciale.\n"
            "6. Ni l'assuré désigné, ni l'employé de l'assuré désigné, ni les personnes suivantes ne doivent "
            "être propriétaires du véhicule ou en possession du véhicule en vertu d'un contrat écrit similaire "
            "à une hypothèque, une vente conditionnelle ou un contrat de location :\n"
            "\t un actionnaire, membre ou associé de l'assuré désigné;\n"
            "\t une personne qui a le même domicile que l'assuré désigné ou que l'employé, l'actionnaire, "
            "le membre ou l'associé de l'assuré désigné.\n"
            "Cette 6e condition ne s'applique pas en cas de location pour un usage qui n'est pas habituel ou "
            "fréquent.\n"
            f"{NEW_PAGE_TAG}\n"
        )

        self.assertEqual(expected, actual)



from risc_generator.content import fr, en
from tests.domain.contract.contract_base import ContractTestBase

INCLUDE = "include"
EXCLUDE = "exclude"


class ContractDeclarationTest(ContractTestBase):
    def test_givenAFRContract_whenGenerateDivulgationText_thenReturnProperFRText(self):
        actual = fr.declaration_articles.generate_divulgation_text()

        a_website = "assureur.ca"
        a_insurer_name = "Assureur"
        a_insurer_address = "221 B Baker Street\nVille (Province)\nA1A 1A1"

        expected = (
            "DIVULGATION\n\n"
            "AVIS CONCERNANT LA PROTECTION DES RENSEIGNEMENTS PERSONNELS\n"
            f"{a_insurer_name} traite de façon confidentielle les renseignements personnels "
            f"qu'elle détient sur vous.\n"
            f"Objet de votre dossier\n"
            f"Nous recueillons et utilisons les renseignements personnels qui vous concernent dans le but de "
            f"gérer votre dossier d'assurance de dommages.\n"
            f"Sécurité\n"
            f"Vos renseignements personnels sont conservés à nos bureaux et protégés par des standards de "
            f"sécurité élevés. Seuls nos employés, mandataires, partenaires de distribution et fournisseurs "
            f"de services peuvent y avoir accès et ce, uniquement lorsque requis par leurs fonctions, mandats "
            f"ou contrats.\n"
            f"{a_insurer_name} peut faire affaires avec un (ou des) fournisseur(s) de services basés "
            f"à l'extérieur du Québec. Ainsi, il est possible que certains de vos renseignements personnels "
            f"détenus par {a_insurer_name} puissent être hébergés à l'extérieur du Québec et être régis par les "
            f"lois applicables à des pays ou états étrangers.\n"
            f"Accès et rectification\n"
            f"Pour avoir accès à votre dossier ou en demander la rectification, faites-en la "
            f"demande à l'adresse suivante :\n"
            f"{a_insurer_name}\n"
            f"a/s du Service d'accès à l'information\n"
            f"{a_insurer_address}\n"
            f"Offre de service\n"
            f"Il se peut que {a_insurer_name}, ses filiales et représentants autorisés "
            f"utilisent vos renseignements nominatifs pour vous informer des produits et services "
            f"susceptibles de vous intéresser. Si toutefois vous ne désirez pas recevoir ce type "
            f"d'information, écrivez-nous à l'adresse ci-dessus.\n"
            f"Pour plus d'informations sur nos pratiques en matière de protection des renseignements "
            f"personnels, consultez notre Énoncé de confidentialité des renseignements personnels sur "
            f"{a_website}/fr/protection-renseignements-personnels.\n"
            f"Votre agent offre exclusivement les produits de {a_insurer_name} et il "
            f"ne reçoit aucune commission.\n"
        )

        self.assertEqual(expected, actual)

    def test_givenAENContract_whenGenerateDivulgationText_thenReturnProperENText(self):
        actual = en.declaration_articles.generate_divulgation_text()

        a_website = "assureur.ca"
        a_insurer_name = "Assureur"
        a_insurer_address = "221 B Baker Street\nVille (Province)\nA1A 1A1"

        expected = (
            "DISCLOSURE\n\n"
            "NOTICE CONCERNING THE PROTECTION OF PERSONAL INFORMATION\n"
            f"{a_insurer_name} protects the confidentiality of your personal information.\n"
            f"PURPOSE OF YOUR FILE\n"
            f"We collect and use your personal information to manage your Property and Casualty insurance file.\n"
            f"SECURITY\n"
            f"Your personal information is stored at our offices and protected by high security measures. Only "
            f"our employees, mandataries, distribution partners and service providers may access your personal "
            f"information, and solely when such access is required to perform their duties, carry out their "
            f"mandate or fulfill their service contract.\n"
            f"{a_insurer_name} may do business with one or more service providers based outside of Quebec. "
            f"It is therefore possible that some of your personal information held by {a_insurer_name} may be "
            f"stored outside of Quebec and governed by the laws of foreign countries or states.\n"
            f"ACCESS AND CORRECTION\n"
            f"To access your file or make a correction to it, send your request in writing to the following address:\n"
            f"{a_insurer_name}\n"
            f"c/o Access to Information Service\n"
            f"{a_insurer_address}\n"
            f"SERVICE OFFERING\n"
            f"{a_insurer_name}, its subsidiaries and their authorized representatives may use your personal "
            f"information to inform you of products and services that may be of interest to you. If, however, "
            f"you do not wish to receive this type of information, please write to us at the address above.\n"
            f"For more information about our personal information protection practices, refer to our personal "
            f"information protection statement at {a_website}/en/personal-information-protection.\n"
            f"Your agent offers {a_insurer_name} products on an exclusive basis and does not receive any commission.\n"
        )

        self.assertEqual(expected, actual)



import random

from faker import Faker

from ...domain.protections import Protections
from ...faker.protections_faker.liability_coverage_faker import (
    liability_coverage_faker,
)
from ...faker.protections_faker.property_coverage_faker import (
    property_coverage_faker,
)
from ...faker.protections_faker.tools.endorsement_coverage_parsing import (
    endorsement_coverage_parsing,
)
from ...faker.protections_faker.tools.protections_faker_validation_tools import (
    is_valid_protections,
    is_only_liability_coverage,
    include_b1_coverage,
    include_b2_coverage,
    include_b3_coverage,
    include_b4_coverage,
)
from ...model.synthetic_protections_model import SyntheticProtectionsModel


class ProtectionsFaker(Faker):
    def __init__(self, model: SyntheticProtectionsModel) -> None:
        """
        model (model): A model to generate fake protections.
        """
        super().__init__()
        self.model = model

    def protections(self, can_have_faq_41: bool) -> Protections:
        """
        can_have_faq_41 (bool): Either or not the protections can include the endorsement FAQ41 in the protections.

        Return:
            A set of protections.
        """
        data_invalid = True
        synthetic_coverage = None

        while data_invalid:
            synthetic_coverage = self.model.sample(num_sample=1)
            if is_valid_protections(synthetic_coverage, can_have_faq_41=can_have_faq_41):
                data_invalid = False

        protections = {}

        liability_coverage = liability_coverage_faker(
            is_only_liability_coverage=is_only_liability_coverage(synthetic_coverage)
        )
        protections.update(liability_coverage)

        property_coverage = property_coverage_faker(
            include_b1_coverage=include_b1_coverage(synthetic_coverage),
            include_b2_coverage=include_b2_coverage(synthetic_coverage),
            include_b3_coverage=include_b3_coverage(synthetic_coverage),
            include_b4_coverage=include_b4_coverage(synthetic_coverage),
        )
        protections.update(property_coverage)

        endorsement_protection = endorsement_coverage_parsing(synthetic_coverage)
        protections.update(**endorsement_protection)

        return Protections(protections=protections)

    def seed_instance(self, seed=42) -> None:
        """
        seed (int): The seed value to use for the generation seed. By default, 42.
        """
        super().seed_instance(seed)
        random.seed(seed)



def change_to_snake_case(string: str) -> str:
    """
    Function to change a text in any case type (e.g. CamelCase) to snake_case.

    string (str): The string to convert to snake_case.

    Return:
        The string in snake_case.
    """
    return ''.join(['_' + i.lower() if i.isupper() else i for i in string]).lstrip('_')


def change_to_camel_case(string: str) -> str:
    """
    Function to change a text in any case type (e.g. snake_case) to CamelCase.

    string (str): The string to convert to CamelCase.

    Return:
        The string in CamelCase.
    """
    first, *others = string.split('_')
    return ''.join([first.capitalize(), *map(str.title, others)])



import random


class DriverLicenseSuspensionFaker:
    def __init__(
        self,
        total_alcohol_suspension: int = 14273,
        total_points_suspension: int = 93047,
        total_drivers: int = 5528681,
    ):
        """
        Faker to generate a probabilistic driver license suspension using the ratio of
        alcohol suspension and points suspension. Default are the 2019 SAAQ values
        https://saaq.gouv.qc.ca/fileadmin/documents/publications/donnees-statistiques-2019.pdf.
        We use the 2019 value due to the COVID restrictions during 2020-2021 where license suspension have
        significantly dropped due to less opportunity to drive, thus less opportunity to receive a driving suspension.

        total_alcohol_suspension: Number of alcohol suspension.
        total_points_suspension: Number of suspension due to points.
        total_drivers: Total diver population.
        """
        self.total_drivers = total_drivers
        self.total_alcohol_suspension = total_alcohol_suspension
        self.total_points_suspension = total_points_suspension

        self.proportion_of_suspension = (total_alcohol_suspension + total_points_suspension) / total_drivers

    def suspension(self) -> bool:
        """
        Method to fake a suspension.

        Return:
            A suspension status.
        """
        suspension = False
        u = random.uniform(0, 1)
        if u <= self.proportion_of_suspension:
            suspension = True
        return suspension



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq16_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Formulaire d'avenant du Québec\n"
        "F.A.Q. N 16\n"
        "Suspension de garanties lors du remisage du véhicule\n"
        "Le titre de l'avenant doit être écrit à la section « Conditions particulières » du contrat "
        "d'assurance. Quant aux informations requises dans l'avenant, elles peuvent être écrites à cette "
        "section ou dans l'avenant même, au choix de l'assureur.\n"
        "Nom de l'assureur : \n"
        "Nom de l'assuré désigné : \n"
        "Avenant à la police d'assurance automobile N : \n"
        "Date de prise d'effet : cet avenant s'applique à partir du à 0 h 01, heure normale "
        "à l'adresse de l'assuré désigné.\n"
        "Date du remisage : à 0 h 01, heure normale à l'adresse de l'assuré désigné\n"
        "Véhicule visé : cet avenant s'applique uniquement au véhicule désigné suivant :\n"
        "(numéro de référence écrit à la section « Conditions particulières » du contrat d'assurance)\n"
        "Description de l'avenant\n"
        "Cet avenant suspend certaines garanties du contrat d'assurance en raison du remisage du véhicule visé.\n"
        "Suspension des garanties\n"
        "L'assuré désigné déclare que le véhicule visé sera retiré de la circulation et remisé à la date du "
        "remisage. Il demande donc qu'à partir de cette date, les garanties suivantes soient suspendues :\n"
        "1. Garanties suspendues au chapitre A\n"
        "Les garanties du chapitre A du contrat d'assurance sont suspendues en ce qui concerne la conduite ou "
        "l'usage des véhicules suivants :\n"
        "\t le véhicule visé;\n"
        "\t tout véhicule dont l'assuré désigné est nouvellement propriétaire qui remplace le véhicule "
        "visé ou qui s'y ajoute;\n"
        "\t tout véhicule de remplacement temporaire qui remplace le véhicule visé.\n"
        "L'assuré désigné continue de bénéficier des autres garanties du chapitre A.\n"
        "2. Garanties suspendues au chapitre B\n"
        "Les garanties des Protections 1 et 2 du chapitre B du contrat d'assurance sont suspendues en ce qui "
        "concerne la conduite des véhicules suivants :\n"
        "\t le véhicule visé;\n"
        "\t tout véhicule dont l'assuré désigné est nouvellement propriétaire qui remplace le véhicule "
        "visé ou qui s'y ajoute;\n"
        "\t tout véhicule de remplacement temporaire qui remplace le véhicule visé.\n"
        "L'assuré désigné continue de bénéficier des autres garanties du chapitre B.\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "Fin de la suspension des garanties\n"
        "Les garanties suspendues sont remises en vigueur à l'une des dates suivantes :\n"
        "\t à la date déterminée par l'assuré désigné, à condition qu'il en ait d'abord informé l'assureur;\n"
        "si aucune date n'est ainsi déterminée, au 1er avril qui suit la date du remisage.\n"
        "Le contrat d'assurance doit être en vigueur pour que les garanties puissent être remises en vigueur.\n"
        "Ristourne\n"
        "L'assuré désigné a droit à une ristourne pour la période de remisage, calculée sur la base suivante :\n"
        "Toutes les autres conditions du contrat d'assurance restent les mêmes.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



from ..config import (
    GENERAL_FAKE_PHONE_NUMBER,
    GENERAL_FAKE_PHONE_NUMBER_1_800,
    FAKE_FAX_PHONE_NUMBER,
    FAKE_EMAIL,
    CLAIMS_FAKE_PHONE_NUMBER,
    CLAIMS_FAKE_PHONE_NUMBER_1_800,
    FAKE_LEGAL_HELP_PHONE_NUMBER,
    FAKE_INSURER_ADDRESS,
    FAKE_INSURER_WEBSITE,
    NEW_PAGE_TAG,
)


def generate_header_text(self) -> str:
    content_text = (
        f"{self.formatted_text_date()}\n\n\n"
        f"ÉMISSION\n"
        f"Numéro client: {self.client_number}\n"
        f"Police / Durée du terme:\n"
        f"{self.policy} / 12 mois\n\n"
        f"JAMAIS SEUL POUR VOS ASSURANCES\n\n"
        f"Vous trouverez vos documents d'assurance aux pages suivantes.\n"
        f"Nous vous suggérons de les lire attentivement afin de vous assurer qu'ils sont conformes.\n"
        f"Si une page Instructions importantes fait partie de ces documents, veuillez y porter une attention "
        f"particulière, puisqu'elle comporte des renseignements essentiels au sujet de vos protections.\n"
        f"N'hésitez pas à communiquer avec nous pour toute question ou commentaire.\n\n"
        f"NOUS JOINDRE\n"
        f"Service à la clientèle\n"
        f"{GENERAL_FAKE_PHONE_NUMBER}\n"
        f"{GENERAL_FAKE_PHONE_NUMBER_1_800}\n"
        f"Télécopieur: {FAKE_FAX_PHONE_NUMBER}\n"
        f"{FAKE_EMAIL}\n"
        f"8 h à 20 h du lundi au vendredi\n"
        f"8 h 30 à 16 h le samedi\n"
        f"Réclamations\n"
        f"{CLAIMS_FAKE_PHONE_NUMBER}\n"
        f"{CLAIMS_FAKE_PHONE_NUMBER_1_800}\n"
        f"Urgence 7 jours/24 heures\n"
        f"Assistance juridique\n"
        f"{FAKE_LEGAL_HELP_PHONE_NUMBER}\n"
        f"8 h 30 à 19 h du lundi au jeudi\n"
        f"8 h 30 à 17 h le vendredi\n"
        f"Votre succursale\n"
        f"{FAKE_INSURER_ADDRESS}\n"
        f"8 h 30 à 17 h du lundi au vendredi\n"
        f"{FAKE_INSURER_WEBSITE}\n"
    )
    if self.payment_method == "facture":
        # Case where payment is a bill, thus it include a billing section in the TDM
        content_text += (
            f"Vos documents d'assurance\n"
            f"(Table des matières)\n"
            f"Instructions importantes\n"
            f"Assurance automobile\n"
            f"Numéro police: {self.policy}\n"
            f"{self.vehicle.format_vehicle_make_year_details()}\n"
            f"Certificats d'assurance\n"
            f"(Documents à conserver)\n"
            f"Facturation\n"
            f"Avantages Client\n"
            f"(Description de vos bénéfices client)\n"
            f"Libellés et avenants\n"
            f"{NEW_PAGE_TAG}\n"
            f"{NEW_PAGE_TAG}\n"  # An empty page
        )
    else:
        # Case where payment is pre-authorized, thus no billing in the documents.
        content_text += (
            f"Vos documents d'assurance\n"
            f"(Table des matières)\n"
            f"Instructions importantes\n"
            f"Assurance automobile\n"
            f"Numéro police: {self.policy}\n"
            f"{self.vehicle.format_vehicle_make_year_details()}\n"
            f"Certificats d'assurance\n"
            f"(Documents à conserver)\n"
            f"Avantages Client\n"
            f"(Description de vos bénéfices client)\n"
            f"Libellés et avenants\n"
            f"{NEW_PAGE_TAG}\n"
            f"{NEW_PAGE_TAG}\n"  # An empty page
        )
    return content_text



from typing import Dict

from ...faker.protections_faker.protections_amount_faker import (
    liability_coverage_amount_faker,
)


def liability_coverage_faker(is_only_liability_coverage: bool) -> Dict:
    """
    Faker function to generate a liability coverage amount base.

    Our observation shows that liability coverage varies depending on the car's protection.
    If the vehicle is protected under liability coverage, liability amount protection is generally
    speaking only 1M$. Otherwise, the liability
    coverage is more equally distributed if the car is protected with material damage coverage. Thus, the cases are the
    following:

    - Only liability coverage: 95% of clients have 1M$ in liability coverage, and 5% have 2M$.
    - Liability and material damage coverage: 60% of clients have 1M$ in liability coverage, and 40% have 2M$.

    The probabilities are a simplification of our market observation. We haven't conducted an empirical analysis.

    Return:
        A dict with the key 'LiabilityCoverage' and the value of the liability coverage.
    """
    probs = [0.6, 0.4]
    if is_only_liability_coverage:
        probs = [0.95, 0.1]

    liability_coverage_amount = liability_coverage_amount_faker(liability_amount_probs=probs)

    return {"LiabilityCoverage": liability_coverage_amount}



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq33_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Quebec Endorsement Form\n"
        "Q.E.F. N 33\n"
        "Insurance for roadside assistance costs\n"
        "The endorsement heading must be entered in the ''Declarations'' section of the insurance contract."
        "Details required for the endorsement may be entered in the ''Declarations'' section or in the endorsement"
        "itself, at the insurer's option.\n"
        "Name of insurer: \n"
        "Named insured: \n"
        "Endorsement to automobile insurance policy N:\n"
        "Effective date: This endorsement will apply from  at 12:01 A.M. "
        "standard time at the address of the named insured.\n"
        "Additional insurance premium payable:\n"
        "\t Amount payable:  \n"
        "\t Due date:  \n"
        "Specified vehicle: This endorsement will apply only to the following described vehicle:\n"
        "(reference number shown in the ''Declarations'' section of the insurance contract)\n"
        "Endorsement description\n"
        "This endorsement provides that the roadside assistance costs described below and incurred for "
        "the specified vehicle will be reimbursed by the insurer.\n"
        "Roadside assistance costs\n"
        "The costs of the following roadside assistance services will be covered:\n"
        "\t battery boosting;\n"
        "\t door unlocking;\n"
        "\t gas delivery, up to 10 liters;\n"
        "\t towing within a radius of   kilometres (minimum 25 km);\n"
        "\t wheel change.\n"
        "Mechanical repairs and parts or supplies used to repair the specified vehicle are "
        "not covered under this endorsement.\n"
        "Limits of liability\n"
        "\t  $ per event giving rise to roadside assistance charges.\n"
        "\t  $ per policy year.\n"
        "\t   events per policy year.\n"
        "Claim\n"
        "The named insured must justify a claim by submitting to the insurer paid invoices "
        "for the roadside assistance costs.\n"
        "All other conditions of the insurance contract remain the same.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq28_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Quebec Endorsement Form\n"
        "Q.E.F. N 28\n"
        "Limitation of coverage for named drivers\n"
        "The endorsement heading must be entered in the ''Declarations'' section of the insurance contract."
        "Details required for the endorsement may be entered in the ''Declarations'' section or in the endorsement"
        "itself, at the insurer's option.\n"
        "Name of insurer: \n"
        "Named insured: \n"
        "Endorsement to automobile insurance policy N:\n"
        "Effective date: This endorsement will apply from  at 12:01 A.M. "
        "standard time at the address of the named insured.\n"
        "Specified vehicle: This endorsement will apply only to the following described vehicle:\n"
        "(reference number shown in the ''Declarations'' section of the insurance contract)\n"
        "Endorsement description\n"
        "This endorsement limits coverage under the insurance contract when the specified vehicle is, "
        "at the time of the loss, being driven or used by:  (named driver)\n"
        "In such instance, coverage will be limited to the following perils, or to those "
        "perils entered specifically for this endorsement in the ''Declarations'' section of the insurance contract:\n"
        "COVERAGE | PERILS | AMOUNT OF INSURANCE AND DEDUCTIBLE | COVERED/NOT COVERED\n"
        "Section A: Civil liability | Property damage or bodily injury to another person | "
        "Amount of insurance:  $ |  $\n"
        "Section B: Damage to insured vehicles | Protection 1 : ''All perils'', Protection 2 : "
        "Perils of collision and upset, Protection 3: All perils other than collision or upset, "
        "Protection 4: Specific perils | Deductible per loss: $ $ $ $ |      \n"
        "All other conditions of the insurance contract remain the same.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



from typing import Dict

from ...faker.protections_faker.protections_amount_faker import (
    b1_deductible_faker,
    b2_deductible_faker,
    b3_deductible_faker,
    b4_deductible_faker,
)

INCLUDE = "include"
EXCLUDE = "exclude"


def property_coverage_faker(
    include_b1_coverage: bool,
    include_b2_coverage: bool,
    include_b3_coverage: bool,
    include_b4_coverage: bool,
) -> Dict:
    """
    Property coverage faker function of the deductible amount protection and logical of what is included and excluded
    base on the property coverage.

    include_b*_coverage (bool): Either or not a specific chapter coverage (B) is included or not.

    Return:
        A dict of the property coverage where the keys are the coverage and the values are the protections deductible,
        protection amount or, 'include' or 'exclude' coverage.
    """
    car_damage_coverage = {
        "PropertyDamageB1Coverage": INCLUDE,
        "PropertyDamageB2Coverage": INCLUDE,
        "PropertyDamageB3Coverage": INCLUDE,
        "PropertyDamageB4Coverage": INCLUDE,
    }

    # Cases:
    # B1 -> B2, B3 and B4 are 'include' since it is an all coverage
    # B2 only -> all others are 'exclude'
    # B3 only -> B1 and B2 are 'exclude' but B4 is 'include'
    # B4 only -> all others are 'exclude'
    # B2 and B3 -> B1 and B4 are 'include' since B2 and B3 is almost equivalent
    # B2 and B4 -> B1 et B3 are 'exclude'

    # PropertyDamageB1Coverage is B1 (or protection 1): All risk coverage
    # PropertyDamageB2Coverage is B2 (or protection 2): Collision coverage
    # PropertyDamageB3Coverage is B3 (or protection 3): Not collision coverage (fire, glass, etc.)
    # PropertyDamageB4Coverage is B4 (or protection 4): A list of designated risk (e.g. glass, fire, etc.).

    if include_b1_coverage and not include_b2_coverage and not include_b3_coverage and not include_b4_coverage:
        deductible = b1_deductible_faker()
        car_damage_coverage.update({"PropertyDamageB1Coverage": deductible})

    elif include_b2_coverage and not include_b1_coverage and not include_b3_coverage and not include_b4_coverage:
        deductible = b2_deductible_faker()
        car_damage_coverage.update({"PropertyDamageB2Coverage": deductible})

        car_damage_coverage.update({"PropertyDamageB1Coverage": EXCLUDE})
        car_damage_coverage.update({"PropertyDamageB3Coverage": EXCLUDE})
        car_damage_coverage.update({"PropertyDamageB4Coverage": EXCLUDE})

    elif include_b3_coverage and not include_b1_coverage and not include_b2_coverage and not include_b4_coverage:
        deductible = b3_deductible_faker()
        car_damage_coverage.update({"PropertyDamageB3Coverage": deductible})

        car_damage_coverage.update({"PropertyDamageB1Coverage": EXCLUDE})
        car_damage_coverage.update({"PropertyDamageB2Coverage": EXCLUDE})

    elif include_b4_coverage and not include_b1_coverage and not include_b2_coverage and not include_b3_coverage:
        deductible = b4_deductible_faker()
        car_damage_coverage.update({"PropertyDamageB4Coverage": deductible})

        car_damage_coverage.update({"PropertyDamageB1Coverage": EXCLUDE})
        car_damage_coverage.update({"PropertyDamageB2Coverage": EXCLUDE})
        car_damage_coverage.update({"PropertyDamageB3Coverage": EXCLUDE})

    elif include_b2_coverage and include_b3_coverage and not include_b1_coverage and not include_b4_coverage:
        deductible = b2_deductible_faker()
        car_damage_coverage.update({"PropertyDamageB2Coverage": deductible})

        deductible = b3_deductible_faker()
        car_damage_coverage.update({"PropertyDamageB3Coverage": deductible})

    elif include_b2_coverage and include_b4_coverage and not include_b1_coverage and not include_b3_coverage:
        deductible = b2_deductible_faker()
        car_damage_coverage.update({"PropertyDamageB2Coverage": deductible})

        deductible = b4_deductible_faker()
        car_damage_coverage.update({"PropertyDamageB4Coverage": deductible})

        car_damage_coverage.update({"PropertyDamageB1Coverage": EXCLUDE})
        car_damage_coverage.update({"PropertyDamageB3Coverage": EXCLUDE})

    return car_damage_coverage



from typing import Dict

INCLUDE = "include"
EXCLUDE = "exclude"


def endorsement_coverage_parsing(synthetic_coverage: Dict) -> Dict:
    """
    Function to parse the synthetic coverage protections to modify the endorsement (e.g. "FAQ") into 'include' value
    if 1 and in 'exclude' value otherwise (0 value).

    synthetic_coverage (Dict): The synthetic coverage dict coverage with 1 for "protected" and 0 otherwise.

    Return:
        A parsed synthetic coverage where endorsement (FAQ) with ones are now 'include' and endorsement
        with zeros are 'exclude'.
    """
    return {key: INCLUDE if value == 1 else EXCLUDE for key, value in synthetic_coverage.items() if "FAQ" in key}



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq41_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Formulaire d'avenant du Québec\n"
        "F.A.Q. N 41\n"
        "Modification aux franchises\n"
        "(Chapitre B)\n"
        "Le titre de l'avenant doit être écrit à la section « Conditions particulières » du contrat "
        "d'assurance. Quant aux informations requises dans l'avenant, elles peuvent être écrites à "
        "cette section ou dans l'avenant même, au choix de l'assureur.\n"
        "Nom de l'assureur : \n"
        "Nom de l'assuré désigné : \n"
        "Avenant à la police d'assurance automobile N : \n"
        "Date de prise d'effet : cet avenant s'applique à partir du à 0 h 01, heure normale "
        "à l'adresse de l'assuré désigné.\n"
        "Véhicule visé : cet avenant s'applique uniquement au véhicule désigné suivant :\n"
        "(numéro de référence écrit à la section « Conditions particulières » du contrat d'assurance)\n"
        "Description de l'avenant\n"
        "Cet avenant apporte les modifications suivantes aux franchises du chapitre B, écrites à l'article "
        "4 de la section « Conditions particulières » du contrat d'assurance :\n"
        "Toutes les autres conditions du contrat d'assurance restent les mêmes.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



# pylint: disable=unnecessary-dunder-call


from unittest import TestCase

from risc_generator import Protections

INCLUDE = "include"
EXCLUDE = "exclude"


class ProtectionsTest(TestCase):
    def test_GivenASetOfProtection_whenSTR_thenReturnProperText(self):
        # Case 1
        dict_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": EXCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
            "FAQ5ACoverage": INCLUDE,
        }
        protections = Protections(protections=dict_protections)

        actual = protections.__str__()
        expected = (
            "Chapter A: 1000000$\nChapter B: \nCollision protected with 250$ franchise\nOther non "
            "collision protected with 250$ franchise"
        )
        self.assertEqual(expected, actual)

        # Case 2
        dict_protections = {
            "LiabilityCoverage": 2000000,
            "PropertyDamageB1Coverage": EXCLUDE,
            "PropertyDamageB2Coverage": 500,
            "PropertyDamageB3Coverage": 250,
            "PropertyDamageB4Coverage": EXCLUDE,
            "FAQ5ACoverage": INCLUDE,
        }
        protections = Protections(protections=dict_protections)

        actual = protections.__str__()
        expected = (
            "Chapter A: 2000000$\nChapter B: \nCollision protected with 500$ franchise\nOther non "
            "collision protected with 250$ franchise"
        )
        self.assertEqual(expected, actual)

        # Case 3
        dict_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": EXCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": EXCLUDE,
            "PropertyDamageB4Coverage": 250,
            "FAQ5ACoverage": INCLUDE,
        }
        protections = Protections(protections=dict_protections)

        actual = protections.__str__()
        expected = (
            "Chapter A: 1000000$\nChapter B: \nCollision protected with 250$ franchise\nDesignated "
            "protected with 250$ franchise"
        )
        self.assertEqual(expected, actual)

    def test_givenASetOfProtectionsWithProtectionNameInProtection_whenIsProtected_thenReturnTrue(
        self,
    ):
        dict_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": EXCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": EXCLUDE,
            "PropertyDamageB4Coverage": 250,
            "FAQ5ACoverage": INCLUDE,
        }
        protections = Protections(protections=dict_protections)

        protected = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB4Coverage": 250,
            "FAQ5ACoverage": INCLUDE,
        }

        for protection_name, _ in protected.items():
            self.assertTrue(protections.is_protected(protection_name))

        not_protected = {
            "PropertyDamageB1Coverage": EXCLUDE,
            "PropertyDamageB3": EXCLUDE,
        }

        for protection_name, _ in not_protected.items():
            self.assertFalse(protections.is_protected(protection_name))

    def test_givenASetOfProtectionsWithProtectionNameInProtection_whenPropertyDamageProtections_thenReturnCorrectList(
        self,
    ):
        dict_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": EXCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": EXCLUDE,
            "PropertyDamageB4Coverage": 250,
            "FAQ5ACoverage": INCLUDE,
        }
        protections = Protections(protections=dict_protections)

        expected_number_of_element_in_property_damage_protections = 2
        actual_number_of_element = len(protections.property_damage_protections())
        self.assertEqual(expected_number_of_element_in_property_damage_protections, actual_number_of_element)

    def test_givenAProtection_whenGetWithCamelCaseProtectionName_returnProtection(self):
        dict_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": EXCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": EXCLUDE,
            "PropertyDamageB4Coverage": 250,
            "FAQ5ACoverage": INCLUDE,
        }
        protections = Protections(protections=dict_protections)

        self.assertIsNotNone(protections.get("LiabilityCoverage"))

    def test_givenAProtection_whenEndorsementProtections_returnAllEndorsementInCamelCase(self):
        dict_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": EXCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": EXCLUDE,
            "PropertyDamageB4Coverage": 250,
            "FAQ5ACoverage": INCLUDE,
        }
        protections = Protections(protections=dict_protections)

        actual = protections.endorsement_protections()

        expected_len = 1
        self.assertEqual(len(actual), expected_len)
        expected_element = ["FAQ5ACoverage"]
        self.assertEqual(actual, expected_element)

        dict_protections = {
            "LiabilityCoverage": 1000000,
            "PropertyDamageB1Coverage": EXCLUDE,
            "PropertyDamageB2Coverage": 250,
            "PropertyDamageB3Coverage": EXCLUDE,
            "PropertyDamageB4Coverage": 250,
            "FAQ5ACoverage": INCLUDE,
            "FAQ23ACoverage": INCLUDE,
            "FAQ55Limitation": INCLUDE,
        }
        protections = Protections(protections=dict_protections)

        actual = protections.endorsement_protections()

        expected_len = 3
        self.assertEqual(len(actual), expected_len)
        expected_element = ["FAQ5ACoverage", "FAQ23ACoverage", "FAQ55Limitation"]
        self.assertEqual(actual, expected_element)



from typing import Dict


def is_valid_protections(synthetic_coverage: Dict, can_have_faq_41: bool) -> bool:
    """
    Function to validate if a set of synthetic coverage is valid based on the Quebec insurance market.

    Conditions are:
        - It must contain the mandatory liability coverage.
        - It does not include the B1 coverage and any other B[2-4] coverage (B1 is a superset of all other
            protections).
        - It does not include B3 and B4 (B3 is a superset of B4).
        - It does not include a FAQ41 if it cannot have one (can_have_faq_41).
        - If it includes a FAQ43 it include a Section B coverage (B1, B2, B3 or B4).

    synthetic_coverage (Dict): A synthetic coverage dict with zeros and ones for the protection configuration.
    can_have_faq_41 (bool): Either or not the protection configuration can include a FAQ41 endorsement (depends on
        driver claims history in the past five years).

    Return:
        A bool representing if the protections is valid.
    """
    # The liability coverage is mandatory in Quebec
    if synthetic_coverage.get("LiabilityCoverage") == 0:
        return False
    valid_protections = True

    # Case where the protection include a Section B-1 (all risks) and other Section B coverage, which
    # is impossible since B-1 cover all loss of B-2 and B-3 (and B-4).
    if include_b1_coverage(synthetic_coverage) and not (
        not include_b2_coverage(synthetic_coverage)
        and not include_b3_coverage(synthetic_coverage)
        and not include_b4_coverage(synthetic_coverage)
    ):
        valid_protections = False

    # Case where the protection include a Section B-3 (all risks) and a B-4 coverage. B-4 is an under set of B-3.
    # i.e. not logical to have both.
    if include_b3_coverage(synthetic_coverage) and include_b4_coverage(synthetic_coverage):
        valid_protections = False

    # If the clients cannot have a FAQ41 (i.e. no claims in the past five years) but the synthetic protection include a
    # FAQ 41.
    if not can_have_faq_41 and synthetic_coverage.get("FAQ41Coverage") == 1:
        valid_protections = False

    # If the protections include a FAQ43, it must have a Section B coverage.
    if synthetic_coverage.get("FAQ43Coverage") and not (
        include_b1_coverage(synthetic_coverage)
        or include_b2_coverage(synthetic_coverage)
        or include_b3_coverage(synthetic_coverage)
        or include_b4_coverage(synthetic_coverage)
    ):
        valid_protections = False

    return valid_protections


def is_only_liability_coverage(synthetic_coverage: Dict) -> bool:
    """
    Function to validate if a synthetic coverage include only a liability coverage and no property damage coverage
    (B1 to B4).

    synthetic_coverage (Dict): A synthetic coverage dict with zeros and ones for the protection configuration.

    Return:
        Either or not the coverage include only liability coverage (no property damage coverage).
    """
    only_liability_coverage = True

    # Not an only liability coverage if at least one property coverage is there
    if (
        include_b1_coverage(synthetic_coverage)
        or include_b2_coverage(synthetic_coverage)
        or include_b3_coverage(synthetic_coverage)
        or include_b4_coverage(synthetic_coverage)
    ):
        only_liability_coverage = False
    return only_liability_coverage


def include_b1_coverage(synthetic_coverage: Dict) -> bool:
    """
    Function to validate if a synthetic coverage include a B1 property damage coverage.

    synthetic_coverage (Dict): A synthetic coverage dict with zeros and ones for the protection configuration.

    Return:
        Either or not the coverage include a B1 property damage coverage.
    """
    return synthetic_coverage.get("PropertyDamageB1Coverage") == 1


def include_b2_coverage(synthetic_coverage: Dict) -> bool:
    """
    Function to validate if a synthetic coverage include a B2 property damage coverage.

    synthetic_coverage (Dict): A synthetic coverage dict with zeros and ones for the protection configuration.

    Return:
        Either or not the coverage include a B2 property damage coverage.
    """
    return synthetic_coverage.get("PropertyDamageB2Coverage") == 1


def include_b3_coverage(synthetic_coverage: Dict) -> bool:
    """
    Function to validate if a synthetic coverage include a B3 property damage coverage.

    synthetic_coverage (Dict): A synthetic coverage dict with zeros and ones for the protection configuration.

    Return:
        Either or not the coverage include a B3 property damage coverage.
    """
    return synthetic_coverage.get("PropertyDamageB3Coverage") == 1


def include_b4_coverage(synthetic_coverage: Dict) -> bool:
    """
    Function to validate if a synthetic coverage include a B4 property damage coverage.

    synthetic_coverage (Dict): A synthetic coverage dict with zeros and ones for the protection configuration.

    Return:
        Either or not the coverage include a B4 property damage coverage.
    """
    return synthetic_coverage.get("PropertyDamageB4Coverage") == 1



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq9_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Quebec Endorsement Form\n"
        "Q.E.F. N 9\n"
        "Marine risk exclusion for amphibious vehicles\n"
        "Modifications\n"
        "The endorsement heading must be entered in the ''Declarations'' section of the insurance contract\n"
        "Name of insurer: \n"
        "Named insured: \n"
        "Endorsement to automobile insurance policy N:\n"
        "Effective date: This endorsement will apply from  at 12:01 A.M. "
        "standard time at the address of the named insured.\n"
        "Specified vehicle: This endorsement will apply only to the following described vehicle:\n"
        "(reference number shown in the ''Declarations'' section of the insurance contract)\n"
        "Endorsement description\n"
        "An amphibious vehicle is a vehicle designed or modified to:\n"
        "\t travel on land, and\n"
        "\t be used in or upon water\n"
        "If the specified vehicle is an amphibious vehicle, this endorsement excludes "
        "coverage under the insurance contract for loss occurring while it is used in or "
        "upon water or while it is being launched into or landed therefrom.\n"
        "All other conditions of the insurance contract remain the same.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



from ..endorsements.endorsement_header import generate_endorsement_page_header_text
from ...config import NEW_PAGE_TAG


@staticmethod
def generate_faq37ab_endorsement_text() -> str:
    endorsement_header_text = generate_endorsement_page_header_text()
    text = (
        f"{endorsement_header_text}"
        "Formulaire d'avenant du Québec\n"
        "F.A.Q. N 37 (A-B)\n"
        "Modifications aux garanties pour les accessoires électroniques\n"
        "(Chapitre B)\n"
        "Le titre de l'avenant doit être écrit à la section « Conditions particulières » du contrat "
        "d'assurance. Quant aux informations requises dans l'avenant, elles peuvent être écrites à "
        "cette section ou dans l'avenant même, au choix de l'assureur.\n"
        "Nom de l'assureur : \n"
        "Nom de l'assuré désigné : \n"
        "Avenant à la police d'assurance automobile N :\n"
        "Date de prise d'effet : cet avenant s'applique à partir du à 0 h 01, heure normale "
        "à l'adresse de l'assuré désigné.\n"
        "Véhicule visé : cet avenant s'applique uniquement au véhicule désigné suivant :\n"
        "(numéro de référence écrit à la section « Conditions particulières » du contrat d'assurance)\n"
        "Description de l'avenant\n"
        "Selon l'option applicable, cet avenant modifie les garanties du chapitre B du contrat "
        "d'assurance pour les « accessoires électroniques » qui ne sont pas des « accessoires "
        "électroniques d'origine ».\n"
        "Option A : Limitation de l'indemnité\n"
        "En cas de sinistre couvert, l'indemnité du chapitre B se limite à …………….. $ pour l'ensemble "
        "des « accessoires électroniques » endommagés qui ne sont pas des « accessoires électroniques "
        "d'origine ».\n"
        "De plus, l'indemnité ne peut jamais dépasser la « valeur au jour du sinistre » des « "
        "accessoires électroniques » endommagés.\n"
        "Cette limitation s'applique même lorsqu'un F.A.Q. N 43 (A à F) intitulé « Modification à "
        "l'indemnisation » est annexé au contrat d'assurance.\n"
        "Option B : Exclusion de la garantie\n"
        "L'ensemble des « accessoires électroniques » qui ne sont pas des « accessoires "
        "électroniques d'origine » sont exclus des garanties du chapitre B.\n"
        "Définitions\n"
        "1. Accessoires électroniques\n"
        "L'expression « accessoires électroniques » utilisée dans cet avenant vise les appareils "
        "électroniques :\n"
        "qui sont fixés ou destinés à être fixés en permanence au véhicule visé; et\n"
        "qui servent à la communication, à la reproduction ou à l'enregistrement du son ou de l'image, "
        "ou des deux à la fois. Cela inclut, entre autres :\n"
        "- les assistants de navigation personnels (GPS);\n"
        "- les lecteurs de disques compacts et de DVD;\n"
        f"{NEW_PAGE_TAG}\n"
        f"{endorsement_header_text}"
        "- les radios, y compris les radios C.B. et les émetteurs-récepteurs amateurs ou autres; et\n"
        "les systèmes de son.\n"
        "2. Accessoires électroniques d'origine\n"
        "L'expression « accessoires électroniques d'origine » utilisée dans cet avenant vise les "
        "accessoires électroniques décrits ci-dessus :\n"
        "\t qui ont été installés par le fabricant ou le concessionnaire; et\n"
        "\t qui étaient compris dans le prix d'achat original du véhicule visé.\n"
        "Toutes les autres conditions du contrat d'assurance restent les mêmes.\n"
        f"{NEW_PAGE_TAG}\n"
    )

    return text



from scipy.stats import poisson


class ClaimsFaker:
    def __init__(self, mu: float = 0.0452) -> None:
        """
        Faker to generate the number of claims an insuree can have in the past five years.
        The default value, 0.0452 is the average claims' frequency for the past 11 years
        according to the GAA Quebec's data (https://gaa.qc.ca/en/statistics/claims-experience/collision-and-upset/).

        The faker use a poisson distribution to simulate the number of claims.
        mu (float): The frequency of a claim. By default, 0.0452 (base on GAA statistics).
        """

        self.mu = mu

    def claims(self, number_of_years: int = 5) -> int:
        """
        Simulate the number of claims in the past `number_of_years` and return
        the number of claims during those X years.

        number_of_years (int): The number of years of claims to sample. By default, 5 years.

        Return:
            The sum of claims in the past X years.
        """
        return poisson.rvs(mu=self.mu, size=number_of_years).sum()
